//--------------------------------------------
// 2005/11/30 Created by Hiroshi Hashiguchi
// Copyright (c) 2005 Hiroshi Hashiguchi
//--------------------------------------------

//--------------------------------------------
// variables
//--------------------------------------------
var VERSION="1.4.4";
var MIN_WINDOW_WIDTH  = 325;
var MIN_WINDOW_HEIGHT = 120;
var MIN_BACK_HEIGHT   = 300;
var HEIGHT_HEAD = 37;

var viewer_scroll_bar;
var result_scroll_bar;

var infoButton;

var LIST_NUM = 50;

var PKEY_MAILTO="PKEY_MAILTO";
var PKEY_CAPTURE_MODE="PKEY_CAPTURE_MODE";
var PKEY_CAPTURE_DATE="PKEY_CAPTURE_DATE";
var PKEY_OPACITY="PKEY_OPACITY";
var PKEY_BRIGHT_ONFOCUS="PKEY_BRIGHT_ONFOCUS";
var PKEY_DUMMY="PKEY_DUMMY";

// command
var BIN_SCREENCAPTURE = "/usr/sbin/screencapture";
var BIN_RM = "/bin/rm";
var BIN_TEST = "/bin/test";

//--------------------------------------------
// Setup widget
//--------------------------------------------
function setupScrollBar() {

	viewer_scroll_bar = new ScrollManager(
		document.getElementById("viewer_frame"),
		document.getElementById("viewer"),
		document.getElementById("scroll_bar1"),
		document.getElementById("scroll_slider1"),
		document.getElementById("scroll_track1"));

	result_scroll_bar = new ScrollManager(
		document.getElementById("list_frame"),
		document.getElementById("result"),
		document.getElementById("scroll_bar2"),
		document.getElementById("scroll_slider2"),
		document.getElementById("scroll_track2"));
}

function getGenericMemoFilename() {
	return "Generic.memo";
}
function setupMemo() {
	loadStatus();
}

function setMemoTitle(str) {
	document.getElementById("title").innerHTML =
		"<img src=\"images/icon_pulldown-s.png\" id=\"icon_pulldown\"/>" +
		str;
}

function setupFrontSide() {
	setMemoTitle(removeFileExtension(getCurrentFilename()));
	document.getElementById("keyword").setAttribute("placeholder",getLocalizedString("LabelKeyword"));

	setupFileList();

	if (isBrightOnFocus()) {
		setOpacityToBright();
	} else {
		setOpacity(getOpacity());
	}
}

function setupTopMenu()
{
	var title = document.getElementById("title");
	title.style.right = title_right;

	var topmenu = document.getElementById("topmenu");
	topmenu.innerHTML =
		getCaptureMenu() + getSpaceTag() +
		getNewMenu() + getSeparatorTag() +
		getRecentMenu() + getSeparatorTag() +
		getListMenu();
}

function setupFileList()
{
	var select_file = document.getElementById("select_file");

	// delete previous children
	var childNum = select_file.childNodes.length;
	for (var i=0; i < childNum;  i++) {
		var child = select_file.childNodes[0];
		select_file.removeChild(child);
	}

	var fileList = MemoStore.getFileList();
	var currentFile = MemoStore.getFilename();
	var option = null;
	for (var i=0; i < fileList.length; i++) {
		var file = fileList[i];
		option = document.createElement("option");
		option.value = file;
		if (currentFile == file) {
			option.selected = true;
		}
		var desc = removeFileExtension(file);
		option.appendChild(document.createTextNode(desc));
		select_file.appendChild(option);
	}

	// filenew
	option = document.createElement("option");
	option.value = "";
	option.appendChild(document.createTextNode(
		getLocalizedString('LabelNewFile')));
	select_file.appendChild(option);
}

function setupSkinList()
{
	var select_skin = document.getElementById("select_skin");

	// delete previous children
	var childNum = select_skin.childNodes.length;
	for (var i=0; i < childNum;  i++) {
		var child = select_skin.childNodes[0];
		select_skin.removeChild(child);
	}

	var skinList = MemoStore.getSkinList();
	var currentSkin = getSkinName();

	var option = null;
	for (var i=0; i < skinList.length; i++) {
		var skin = skinList[i];
		option = document.createElement("option");
		option.value = skin;
		if (currentSkin == skin) {
			option.selected = true;
		}
		var skinDesc = localizedStrings[ "Skin." + skin];
		option.appendChild(document.createTextNode(skinDesc));
		select_skin.appendChild(option);
	}

}

function setupOpacityList()
{
	var select_opacity = document.getElementById("select_opacity");

	// delete previous children
	var childNum = select_opacity.childNodes.length;
	for (var i=0; i < childNum;  i++) {
		var child = select_opacity.childNodes[0];
		select_opacity.removeChild(child);
	}

	var opacityList = new Array(
		"1.0", "0.9", "0.8", "0.7", "0.6", "0.5");
	var currentOpacity = getOpacity();

	var option = null;
	for (var i=0; i < opacityList.length; i++) {
		var opacity = opacityList[i];
		option = document.createElement("option");
		option.value = opacity;
		if (currentOpacity == opacity) {
			option.selected = true;
		}
		option.appendChild(document.createTextNode((opacity*100) + "%"));
		select_opacity.appendChild(option);
	}

}

function setupBackSide()
{
	document.getElementById("label_filename").innerHTML =
		getLocalizedString("LabelFilename");
	document.getElementById("label_deletecheck").innerHTML =
		getLocalizedString("LabelDeleteCheck");
	document.getElementById("label_mail").innerHTML =
		getLocalizedString("LabelMail");
	document.getElementById("label_capture").innerHTML =
		getLocalizedString("LabelCapture");
	document.getElementById("label_capture2").innerHTML =
		getLocalizedString("LabelCapture2");
	document.getElementById("label_capture3").innerHTML =
		getLocalizedString("LabelCapture3");
	document.getElementById("label_skin").innerHTML =
		getLocalizedString("LabelSkin");
	document.getElementById("label_opacity").innerHTML =
		getLocalizedString("LabelOpacity");
	document.getElementById("label_bright").innerHTML =
		getLocalizedString("LabelBright");

	infoButton  = new InfoButtonAnimator("info_i" , "info_rollie");

	createGenericButton(document.getElementById("button_done"), 
		getLocalizedString('ButtonDone'), commitSetting, 50);

	createGenericButton(document.getElementById("button_cancel"), 
		getLocalizedString('ButtonCancel'), revertSetting, 50);

	document.getElementById("version").innerText = VERSION;

	document.getElementById("back_help").innerHTML =
		"<a href=\"JavaScript:widget.openURL('" + getLocalizedString("URL_HELP") + "');\">" +
		getLocalizedString("LabelHelp") +
		" <img src=\"images/icon_help.png\" class=\"icon_image\"/>" +
		"</a>";
}

function setup() {

	offFocusKeywordBox();
	clearInfo();
	setupScrollBar();

	setupMemo();
	setupFrontSide();
	setupBackSide();
	setupTopMenu();

	setupWidget();
}

function redrawScrollBar() {
	// redraw scroll bar
	if (isModeResult()) {
		result_scroll_bar.redraw();

	} else if (isModeViewer()) {
		viewer_scroll_bar.redraw();

	}
}

var _show_back = false;
function isShowBack() {
	return _show_back;
}
function showBack() {

	// dummy action
	widget.setPreferenceForKey("dummy", PKEY_DUMMY);

	// setup current file preferences
	document.getElementById("text_filename").value =
		removeFileExtension(getCurrentFilename());

	var oldname = getMemoName();
	var oldname_caption = "";
	if (oldname) {
		oldname_caption = "[ " + oldname + " ]";
	}
	document.getElementById("label_oldname").innerText = oldname_caption;

	document.getElementById("checkbox_trash").checked = false;

	var file_preference_disabled = false;
	if (getCurrentFilename() == getGenericMemoFilename()) {
		file_preference_disabled = true;
	}
	document.getElementById("text_filename").disabled = file_preference_disabled;
	document.getElementById("checkbox_trash").disabled = file_preference_disabled;

	// setup global preferences
	var mail_to = widget.preferenceForKey(PKEY_MAILTO);
	if (!mail_to) {
		mail_to = getLocalizedString("LabelUnknown");
	}
	document.getElementById("text_mailto").value = mail_to;

	document.getElementById("checkbox_capture").checked =
		isCaptureWindowMode();

	document.getElementById("checkbox_capture_date").checked =
		isCaptureInsertDate();

	document.getElementById("checkbox_bright").checked =
		isBrightOnFocus();

	// skin
	setupSkinList();

	// opacity
	setupOpacityList();

	// change flip
	var front = document.getElementById("front");
	var back = document.getElementById("back");


	if (window.outerHeight < MIN_BACK_HEIGHT) {
		window.resizeTo(window.outerWidth, MIN_BACK_HEIGHT);
	}
	widget.prepareForTransition("ToBack");

	front.style.display = "none";
	back.style.display  = "block";

	setTimeout("widget.performTransition();", 0);

	infoButton.clearRollie();

	_show_back = true;
}
function showFront() {

	setupFrontSide();

	var front = document.getElementById("front");
	var back = document.getElementById("back");

	widget.prepareForTransition("ToFront");

	if (frontHeight < MIN_BACK_HEIGHT) {
		window.resizeTo(window.outerWidth, frontHeight);
	}
	
	front.style.display = "block";
	back.style.display  = "none";

	listMemo();
	setTimeout("widget.performTransition();", 0);

	_show_back = false;
}

//--------------------------------------------
// Mini management
//--------------------------------------------
var _mini_flag = false;
function isMini() {
	return _mini_flag;
}
function showMini(not_save_flag) {
	document.getElementById("front").style.display = "none";
	document.getElementById("mini").style.display = "block";

	document.getElementById("mini_filename").innerText =
		removeFileExtension(getCurrentFilename());
	_mini_flag = true;

	setupMini();

	if (!not_save_flag) {
		saveStatus();
	}
}
function hideMini(not_save_flag) {
	document.getElementById("front").style.display = "block";
	document.getElementById("mini").style.display = "none";
	_mini_flag = false;

	if (!not_save_flag) {
		saveStatus();
	}
}
function dblClickOnMini(event) {
	hideMini();
	event.stopPropagation();
	event.preventDefault();
}

function setupMini() {
	var menu = document.getElementById("mini_menu");
	menu.innerHTML =
		getCaptureMenu() + getSpaceTag() +
		getNewMenu();
}

function onclickMini(event) {
	showMini();
}
function onclickNormal(event) {
	hideMini();
}


//--------------------------------------------
// Page navigtion
//--------------------------------------------
var _page = 0;
function clearPage() {
	_page = 0;
}
function resetPage() {
	_page = 0;
	displaySubMenu2(getPreviousMenu(isPreviousPage()) +
		" " + getSeparatorTag() + " " +
		getNextMenu(isNextPage()), false);
}
function next() {
	var page = _page + LIST_NUM;
	var total = MemoStore.getMemoCount();
	if (page < total) {
		_page = page;
		result_scroll_bar.resetPosition();
		listMemo(_page, isListRecent());
	}
}
function previous() {
	_page = _page - LIST_NUM;
	if (_page < 0) {
		_page = 0;
	}
	result_scroll_bar.resetPosition();
	listMemo(_page, isListRecent());
}

function isNextPage() {
	return ((_page + LIST_NUM) < MemoStore.getMemoCount());
}
function isPreviousPage() {
	return (_page > 0);
}
function getPage() {
	return _page;
}
function setPage(page) {
	_page = page;
}
//--------------------------------------------
// Memo navigtion
//--------------------------------------------
var _current_seq = 0;
function setCurrentSeq(seq) {
	_current_seq = seq;
}

function getCurrentSeq(seq) {
	return _current_seq;
}
function incCurrentSeq() {
	var maxCount = 0;
	if (isListSearch()) {
		maxCount = getSearchCount();
	} else {
		maxCount = MemoStore.getMemoCount();
	}

	if ((_current_seq+1) < maxCount) {
		_current_seq++;
		return true;
	} else {
		return false;
	}
}
function decCurrentSeq() {
	if ((_current_seq-1) >= 0) {
		_current_seq--;
		return true;
	} else {
		return false;
	}
}

function _navigateMemo(seq) {
	var keyword = null;
	if (isListDefault()) {
		MemoStore.getList(seq, 1);
	} else if (isListRecent()) {
		MemoStore.getRecentList(seq, 1);
	} else if (isListSearch()) {
		keyword = document.getElementById("keyword").value;
		MemoStore.findByKeyword(keyword, seq, 1);
	}
	MemoStore.next();
	var uuid = MemoStore.uuid();
	setCurrentUUID(uuid);
	display(seq, uuid, false, keyword);

}

function nextMemo() {
	var seq = getCurrentSeq();
	var isOK = incCurrentSeq();
	if (isOK) {
		_navigateMemo(++seq);
	}
}
function previousMemo() {
	var seq = getCurrentSeq();
	var isOK = decCurrentSeq();
	if (isOK) {
		_navigateMemo(--seq);
	}
}

//--------------------------------------------
// listing functions
//--------------------------------------------
var _list_mode = 0;
var _search_result = 0;
function setSearchCount(count) {
	_search_count = count;
}
function getSearchCount() {
	return _search_count;
}
function resetSearchCount() {
	_search_count = 0;
}

function isListDefault() {
	return (_list_mode == 0);
}
function isListRecent() {
	return (_list_mode == 1);
}
function isListSearch() {
	return (_list_mode == 2);
}
function list(offset, keyword) {
	var result  = document.getElementById("result");
	var html = ""; 
	
	var title;
	var seq = offset;

	while (MemoStore.next()) {
		title = text2html(MemoStore.title());
		if (keyword != "") {
			title = decorateHitword(title, keyword);
		}

		if (_list_mode == 1) {
			sub_str = transDate2(MemoStore.updatedDate());
		} else {
			sub_str = transDate1(MemoStore.createdDate());
		}
		keyword = keyword.replace(/\\/, "\\\\");
		keyword = keyword.replace(/'/, "\\'");
		html += "<a href=\"JavaScript:display(" + seq + "," +
			"'" +MemoStore.uuid() + "', false, '" + keyword + "');\">" +
			"<span class=\"number\">" + sub_str + "</span>&nbsp;&nbsp;" +
			title + "</a>";

		html += "<br/>";
		seq++;
	}
	result.innerHTML += html;

}

function listMemo(start, recent_flag) {
	if (start === undefined) {
		start = 0;
	}
	var result  = document.getElementById("result");
	result.innerHTML = "";

	var count;
	
	if (recent_flag) {
		count = MemoStore.getRecentList(start, LIST_NUM);
		_list_mode = 1;
	} else {
		count = MemoStore.getList(start, LIST_NUM);
		_list_mode = 0;
	}

	if (count > 0) {
		list(start, "");
	}
	clearCurrentUUID();
	changeModeList(true);
	resetSearchCount();
}

function search(keyword) {
	keyword = keyword.replace(/^ +/, "");
	if (keyword != "") {
		var result  = document.getElementById("result");
		result.innerHTML
			= decorateTitle(getLocalizedString("SearchResult")) + "<br/>";

		var count = MemoStore.findByKeyword(keyword, 0, 50);
		if (count > 0) {
			_list_mode = 2;
			list(0, keyword);
		}
		clearCurrentUUID();
		changeModeList(false);
		setSearchCount(count);

	} else {
		resetPage();
		result_scroll_bar.resetPosition();
		listMemo();
	}
}

function listOnMenu() {
	resetPage();
	result_scroll_bar.resetPosition();
	listMemo();
	clearKeywordBox();
}

function listRecentOnMenu() {
	resetPage();
	result_scroll_bar.resetPosition();
	listMemo(0, true);
	clearKeywordBox();
}

function clearKeywordBox() {
	document.getElementById("keyword").value = "";
}

//--------------------------------------------
// Infomation display
//--------------------------------------------
function displayInfo(str) {

	document.getElementById("footer").innerHTML = str;
}
function clearInfo() {
	//document.getElementById("footer").innerHTML = "";
	document.getElementById("footer").innerHTML =
		getLocalizedString('Count') + " : " + MemoStore.getMemoCount();
}

function displaySubMenu1(str, flag) {
	if (flag) {
		str = "<img src=\"images/icon_arrow-right1.gif\"" +
			" style=\"vertical-align:text-bottom\"/> " + str;
	}
	document.getElementById("submenu").innerHTML = str;
}
function displaySubMenu2(str, flag) {
	if (flag) {
		str = "<img src=\"images/icon_arrow-right2.gif\"" +
		" style=\"vertical-align:text-bottom\"/> " + str;
	}
	document.getElementById("submenu").innerHTML = str;
}
function clearSubMenu() {
	document.getElementById("submenu").innerHTML = "";
}

//--------------------------------------------
// display memo
//--------------------------------------------
function display(seq, uuid, new_flag, keyword) {
	if (uuid) {
		var viewer  = document.getElementById("viewer");
		var editor  = document.getElementById("editor");

		setCurrentSeq(seq);

		MemoStore.findByUUID(uuid);
		MemoStore.next();

		displayInfo(getMemoInformationLine());
		if (new_flag) {
			displaySubMenu1(getMenuSet2());
		} else {
			displaySubMenu1(getMenuSet1());
		}

		var content = "";
		content = MemoStore.content();
		editor.value = content;

		content = text2html(content);
		content = convertURLToLinkTag(content);
		content = convertURLToLinkTag2(content);
		content = convertURLToLinkTag3(content);
		content = convertURLToLinkTag4(content);
		content = convertURLToIMGTag(content);
		content = addIconOnAnchor(content);
		if (keyword) {
			content = decorateHitword(content, keyword);
			content = removeDecorationInAHrefTag(content);
			content = removeDecorationInImgTag(content);
		}

		if (!content.match(/\/n$/m)) {
			content += "<br/>";
		}

		viewer.innerHTML = content + getImageMaskTag();

		setCurrentUUID(uuid);
		changeModeToViewer();

		viewer_scroll_bar.resetPosition();
		redrawScrollBar();
		initImageList();
	}
}

// memo information
// 0 : updated date
// 1 : created date
// 2 : uuid
var memo_info_mode = 0;
var MEMO_INFO_MODE_MAX = 1;
function getMemoInformationLine() {
	var str = "";
	
	var seqNavi = getPreviousMemoMenu() +
		getNextMemoMenu();

	str = "<span class=\"sequence\">" +
		("00000" + (getCurrentSeq()+1)).substr(-5) + "</span> " +
		transDate3(MemoStore.updatedDate());

	return str + "&nbsp;&nbsp;" + seqNavi;
}

function changeMemoInfomationLine() {
	memo_info_mode++;
	if (memo_info_mode > MEMO_INFO_MODE_MAX) {
		memo_info_mode = 0;
	}
	displayInfo(getMemoInformationLine());
}

//--------------------------------------------
// delete/edit/cancel/save/new
//--------------------------------------------
var _is_new;
function setNew() {
	_is_new = true;
}
function resetNew(){
	_is_new = false;
}
function isNew() {
	return _is_new;
}

function del() {
	if (getCurrentUUID()) {
		openDialog(getLocalizedString('MSG_ASK_DELETE'), do_del);
	}
}
function do_del(flag) {
	uuid = getCurrentUUID();
	if (flag && uuid) {
		// delete 
		var str = document.getElementById("editor").value;
		var files = str.match(/@capture{[^}]+}/g);
		for (index in files) {
			var file = files[index];
			var filename = MemoStore.getCapturePath() +
				"/" + file.replace(/@capture{([^}]+)}/, "$1");
			deleteFile(filename);
		}

		// delete data
		MemoStore.del(uuid);
		clearCurrentUUID();
		listMemo();
		resetPage();
	}
}

function edit(uuid) {
	if (!uuid) {
		uuid = getCurrentUUID();
	}
	if (uuid && !isModeEditor()) {
		changeModeToEditor();
	}
	resetModifiedFlag();
	resetNew();
	MemoStore.resetUndo();
}

// create new memo from current memo
function dupmemo() {
	var editor = document.getElementById("editor");
	pasteToEditor("* " + editor.value, true);
	setModifiedFlag();
	backToListFrame();
}

var _icon_copy_flash_timer;
var _icon_copy_flash_count;
function flashCopyIcon() {
	var icon_copy_p = document.getElementById("icon_copy_p");

	var opacity = (_icon_copy_flash_count % 2) / 2 + 0.5;
	icon_copy_p.style.opacity = opacity;

	_icon_copy_flash_count--;
	if (_icon_copy_flash_count <= 0) {
		clearInterval(_icon_copy_flash_timer);
		icon_copy_p.style.opacity = 0.5;
	}
}
function copyToPasteboard(event) {
	var editor = document.getElementById("editor");
	var content = editor.value;
	if (content) {
		MemoStore.copyToPasteboard(content);
		_icon_copy_flash_count = 8;
		_icon_copy_flash_timer = setInterval(flashCopyIcon, 50);
	}
}

function tool() {
	startSlide(tool_callback1, true);
}
function tool_callback1() {
	displaySubMenu1(getMenuSet3());
}

function tool2() {
	startSlide(tool_callback2, false);
}
function tool_callback2() {
	displaySubMenu1(getMenuSet1());
}

function endEdit() {
	if (isModeEditor()) {
		if (getCurrentUUID()) {
			if (isNew()) {
				backToListFrame();
			} else {
				displaySubMenu1(getMenuSet1());
				display(getCurrentSeq(), getCurrentUUID());
			}
		} else {
			backToListFrame();
		}
	}
}
function _getMenuSet(menu) {
	return  getCopyToPasteboardMenu() + "&nbsp;" +"&nbsp;" +
		getDeleteMenu() + getSeparatorTag() +
		getEditMenu() + getSeparatorTag() +
		menu + "&nbsp;" + "&nbsp;" +
		getToolMenu();
}
function getMenuSet1() {
	return _getMenuSet(getBackMenu());
}
function getMenuSet2() {
	return _getMenuSet(getListMenu());
}
function getMenuSet3() {
	return getDupMenu() + getSeparatorTag() +
		getMailMenu() +"&nbsp;" + "&nbsp;" +
		getToolMenu2();
}
function cancelEdit() {
	endEdit();
}

function save(uuid) {
	if (!isModeEditor()) {
		return;
	}

	var viewer  = document.getElementById("viewer");
	var editor = document.getElementById("editor");

	if (!uuid) {
		uuid = getCurrentUUID();
	}
	var content = convertToAppropriateCharacter(editor.value);

	if (uuid) {
		MemoStore.update(content, uuid);
	} else {
		uuid = MemoStore.insert(content);
		setCurrentUUID(uuid);
	}

	listMemo();
	clearEditor();
	clearSubMenu();
}

function newmemo() {
	if (isMini()) {
		hideMini();
	}
	clearCurrentUUID();
	clearEditor();
	changeModeToEditor();
	resetModifiedFlag();
	clearKeywordBox();
	clearPage();
	setNew();
	displayInfo("");
}

function backToListFrame() {
	keyword = document.getElementById("keyword").value;
	if (isModifiedFlag()) {
		if (keyword) {
			search(keyword);
		} else {
			listMemo(getPage(), isListRecent());
		}
	}
	changeModeList(isListRecent()|isListDefault());
}

function mailto() {
	var editor  = document.getElementById("editor");
	var body = editor.value;
	var idx = body.indexOf("\n");
	var mail_to = widget.preferenceForKey(PKEY_MAILTO);
	var subject = "";
	if (idx > 0) {
		subject = body.substr(0, idx);
	}
	if (subject.length > 30) {
		subject = subject.substr(0, 30) + "...";
	}
	location="mailto:" + mail_to +
		"?" +
		"subject=" + subject +
		"&body=" + body;

	/*
	var cmd = "/usr/bin/osascript SendMail.scpt"
	cmd += " " + escape(subject);
	cmd += " " + escape(body);
	cmd += " " + mail_to;
	cmd += " " + "xcatsan@neko.com";

	var id = widget.system(cmd, null);
	debug(id.errorString);
	*/
}
function capture() {
	if (!isCaptureCommandAvailable) {
		return;
	}
	widget.openApplication("com.apple.dock");

	var filename = getDateString() + ".png";
	var fullpath = MemoStore.getCapturePath() + "/" + filename;
	var option;
	if (isCaptureWindowMode()) {
		option = "-ixW";
	} else {
		option = "-ix";
	}
	var cmd = widget.system(
		BIN_SCREENCAPTURE + " " + option + " " + fullpath, null);

	if (isFileExists(fullpath)) {
		var str = "";
		if (isCaptureInsertDate()) {
			str += (new Date()).toLocaleString() + "\n";
		}
		str += "@capture{" + filename + "}\n"

		var new_flag = false;
		if ((document.getElementById("editor").value == "" && isNew())
			|| isMini()) {
			new_flag = true;
		}

		if (isMini()) {
			hideMini();
		}

		pasteToEditor(str, new_flag);

	}
	widget.openApplication("com.apple.dashboardlauncher");
}


//--------------------------------------------
// cut/copy & paste
//--------------------------------------------
function cut(event) {
	var editor = document.getElementById("editor");
	var content = editor.value;
	if (content) {
		event.clipboardData.setData("text/plain", content);
		editor.value = "";
	}
	event.preventDefault();
}

function copy(event) {
	var editor = document.getElementById("editor");
	var content = editor.value;
	if (content) {
		event.clipboardData.setData("text/plain", content);
	}
	event.preventDefault();
}

function paste(event) {
	var str = event.clipboardData.getData("text/plain");
	if (str) {
		pasteToEditor(str);
	}
	event.preventDefault();
}

function pasteToEditor(str, new_flag) {
	var viewer  = document.getElementById("viewer");
	var editor = document.getElementById("editor");

	if (!new_flag) {
		if (getMode() == 2) {
			new_flag = true;
		}
	}

	if (new_flag) {
		// new
		content = convertToAppropriateCharacter(str);
		uuid = MemoStore.insert(content);
		setCurrentUUID(uuid);
		display(0, uuid, true);
		editor.value = str;
	} else {
		// append
		if (!editor.value.match(/\/n$/m)) {
			editor.value += "\n";
		}
		editor.value += str;

		uuid = getCurrentUUID();
		content = convertToAppropriateCharacter(editor.value);
		MemoStore.update(content, uuid);
		display(getCurrentSeq(), uuid);
	}
}

//--------------------------------------------
// Drag & drop
//--------------------------------------------
function dragdrop(event)
{
	var str = null;
	try {
		str = event.dataTransfer.getData("WebURLsWithTitlesPboardType");
		if (str === undefined) {
			str = event.dataTransfer.getData("text/uri-list");
			if (str === undefined) {
				str = event.dataTransfer.getData("text/plain");
			} else {
				if (str.match(/file:\/\//)) {
					str = MemoStore.unescape(str);
				}
			}
		} else {
			str = str.replace(/<[^>]+>/g, '');
			str = str.replace(/\t/g, '');
			str = str.replace(/\n+/mg, "\n");
			str = str.replace(/(^[^\n]+)\n(.+)/mg, "$2\n$1");
			str = str.replace(/^\n/mg, "");
			str2 = MemoStore.urlFromDragPboard();
			if (str && str2 && !str.match(str2)) {
				var alt = str.split("\n");
				str = alt[0] + "\n" + str2;
			}
		}
	} catch(ex) {
	}
	if (str) {
		var new_flag = false;

		if (isMini()) {
			hideMini();
			new_flag = true;
		} else {
			if (event.clientY < HEIGHT_HEAD) {
				new_flag = true;
			}
		}

		pasteToEditor(str, new_flag);
	}

	event.stopPropagation();
	event.preventDefault();
}
function dragenter(event)
{
	event.stopPropagation();
	event.preventDefault();
}
function dragover(event)
{
	event.stopPropagation();
	event.preventDefault();
}
function dragleave(event)
{
	event.stopPropagation();
	event.preventDefault();
}

//--------------------------------------------
// Handle double click
//--------------------------------------------
function dblClick(event) {
	if (isCurrentUUID()) {
		edit();
	} else {
		newmemo();
	}
	event.stopPropagation();
	event.preventDefault();
}
function onClickOnViewer(event) {
	mouseoutOnImage();
}
function dblClickAtList(event) {
	newmemo();
	event.stopPropagation();
	event.preventDefault();
}
function dblClickOnBody(event) {
	if (!isMini()) {
		if (event.clientY < HEIGHT_HEAD) {
			showMini();
		}
	}
}

//--------------------------------------------
// attribute
//--------------------------------------------
var current_uid = "";
function setCurrentUUID(uuid) {
	current_uid = uuid;
}
function getCurrentUUID() {
	return current_uid;
}
function clearCurrentUUID() {
	current_uid = "";
}
function isCurrentUUID() {
	return (current_uid != "");
}

//--------------------------------------------
// handling mode
//--------------------------------------------
var mode = 0;
// 0: viewer
// 1: editor
// 2: result
function changeModeToViewer() {
	mode = 0;
	redrawScrollBar();
	displayViewer();
}
function changeModeToEditor() {
	mode = 1;
	displaySubMenu2(
			getDeleteMenu() + getSeparatorTag() +
			getEndMenu(), false);
	displayEditor();
}
function changeModeList(navigation_flag) {
	mode = 2;
	clearInfo();

	if (navigation_flag) {
		displaySubMenu2(getPreviousMenu(isPreviousPage()) +
			" " + getSeparatorTag() + " " +
			getNextMenu(isNextPage()), false);
	} else {
		clearSubMenu();
	}
	redrawScrollBar();
	displayList();
}
function isModeViewer() {
	return (mode == 0);
}
function isModeEditor() {
	return (mode == 1);
}
function isModeResult() {
	return (mode == 2);
}
function getMode() {
	return mode;
}

//--------------------------------------------
// event handler
//--------------------------------------------
var last_position;
var frontHeight;
function mouseDown(event)
{
	document.addEventListener("mousemove", mouseMove, true);
	document.addEventListener("mouseup", mouseUp, true);

	last_position = {x:event.x, y:event.y};

	event.stopPropagation();
	event.preventDefault();

}
function resizeWidget(newWidth, newHeight) {

	if (newWidth < MIN_WINDOW_WIDTH) {
		newWidth = MIN_WINDOW_WIDTH;
	}
	if (newHeight < MIN_WINDOW_HEIGHT) {
		newHeight = MIN_WINDOW_HEIGHT;
	}
	window.resizeTo(newWidth, newHeight);
	document.getElementById("front").style.height = newHeight;
	frontHeight = newHeight;

	resizeImages();
	redrawScrollBar();
	mouseoutOnImage();
	saveStatus();
}

function resizeWidgetBy(dx, dy) {
	var newWidth  = window.outerWidth  + dx;
	var newHeight = window.outerHeight + dy;
	resizeWidget(newWidth, newHeight);
}

function mouseMove(event)
{
	var dx = 0;
	var dy = 0;
	dx = event.x - last_position.x;
	dy = event.y - last_position.y;
	last_position = {x:event.x, y:event.y};
	
	resizeWidgetBy(dx, dy);

	event.stopPropagation();
	event.preventDefault();

}
function mouseUp(event)
{
	document.removeEventListener("mousemove", mouseMove, true);
	document.removeEventListener("mouseup", mouseUp, true); 
	event.stopPropagation();
	event.preventDefault();
}

function mouseMoveOnFront(event) {
	infoButton.fadeIn();
}

function mouseOutOnFront(event) {
	infoButton.fadeOut();
}

function changeEditor(event) {
	if (!isModeEditor()) {
		return;
	}
	var viewer  = document.getElementById("viewer");
	var editor = document.getElementById("editor");

	uuid = getCurrentUUID();
	var content = convertToAppropriateCharacter(editor.value);

	if (uuid) {
		MemoStore.update(content, uuid);
	} else {
		uuid = MemoStore.insert(content);
		setCurrentUUID(uuid);
	}
	setModifiedFlag();
}

//--------------------------------------------
// Modified flag functions
//--------------------------------------------
var _modified_flag;
function setModifiedFlag() {
	_modified_flag = true;
}
function resetModifiedFlag() {
	_modified_flag = false;
}
function isModifiedFlag() {
	return _modified_flag;
}

//--------------------------------------------
// Control div visiblity
//--------------------------------------------
function displayViewer() {
	document.getElementById("viewer_frame").style.visibility ="visible";
	document.getElementById("list_frame").style.visibility ="hidden";
	document.getElementById("editor_frame").style.visibility       = "hidden";
}

function displayEditor() {
	document.getElementById("viewer_frame").style.visibility ="hidden";
	document.getElementById("list_frame").style.visibility ="hidden";
	document.getElementById("editor_frame").style.visibility      = "visible";

	document.getElementById("editor").focus();
}
function displayList() {
	document.getElementById("viewer_frame").style.visibility ="hidden";
	document.getElementById("list_frame").style.visibility ="visible";
	document.getElementById("editor_frame").style.visibility       ="hidden";
}

//--------------------------------------------
// Mouse wheel handling
//--------------------------------------------
//document.addEventListener("mousewheel", mouseWheel, false);
function mouseWheel(event) {
	if (isModeResult()) {
		result_scroll_bar.wheelScroll(event);
	} else if (isModeViewer()) {
		viewer_scroll_bar.wheelScroll(event);
	}
}

//--------------------------------------------
// button
//--------------------------------------------
function getCaptureMenu() {
	return "<a href=\"JavaScript:capture();\">" +
		"<img src=\"images/icon_capture.png\" id=\"icon_capture\" onmouseover=\"this.style.opacity=0.5;\" onmouseout=\"this.style.opacity=0.2;\" alt=\"" + localizedStrings['Capture'] + "\"/></a>";
}
function getMailMenu() {
	return "<a href=\"JavaScript:mailto();\">" +
		localizedStrings['Mail'] + "</a>";
}
function getEditMenu() {
	return "<a href=\"JavaScript:edit();\">" +
		localizedStrings['Edit'] + "</a>";
}
function getDeleteMenu() {
	return "<a href=\"JavaScript:del();\">" +
		localizedStrings['Delete'] + "</a>";
}
function getSaveMenu() {
	return "<a href=\"JavaScript:save();\">" +
		localizedStrings['Save'] + "</a>";
}
function getCancelMenu() {
	return "<a href=\"JavaScript:cancelEdit();\">" +
		localizedStrings['Cancel'] + "</a>";
}
function getEndMenu() {
	return "<a href=\"JavaScript:endEdit();\">" +
		localizedStrings['End'] + "</a>";
}
function getListMenu() {
	return "<a href=\"JavaScript:listOnMenu();\">" +
		localizedStrings['List'] + "</a>";
}
function getRecentMenu() {
	return "<a href=\"JavaScript:listRecentOnMenu();\">" +
		localizedStrings['Recent'] + "</a>";
}

function getNewMenu() {
	return "<a href=\"JavaScript:newmemo();\">" +
		localizedStrings['New'] + "</a>";
}

function getMenuSeparator() {
	return "&nbsp;<img src=\"images/separator.gif\"" +
		" style=\"vertical-align:text-bottom\"/>&nbsp;";
}

function getBackMenu() {
	return "<a href=\"JavaScript:backToListFrame();\">" +
		localizedStrings['Back'] + "</a>";
}

function getDupMenu() {
	return "<a href=\"JavaScript:dupmemo();\">" +
		localizedStrings['Copy'] + "</a>";
}

function getCopyToPasteboardMenu() {
	return "<a href=\"JavaScript:copyToPasteboard();\">" +
		"<img src=\"images/icon_copy_p.png\" id=\"icon_copy_p\" onmouseover=\"this.style.opacity=1.0\" onmouseout=\"this.style.opacity=0.5\" alt=\"" + localizedStrings['Copy'] + "\"/></a>";
}

function getNextMemoMenu() {
	return "<a href=\"JavaScript:nextMemo();\">" +
		"<img src=\"images/icon_next_memo.png\" class=\"icon_memo_navi\" onmouseover=\"this.style.opacity=1.0\" onmouseout=\"this.style.opacity=0.5\" alt=\"" + localizedStrings['NextMemo'] + "\"/></a>";
}

function getPreviousMemoMenu() {
	return "<a href=\"JavaScript:previousMemo();\">" +
		"<img src=\"images/icon_prev_memo.png\" class=\"icon_memo_navi\" onmouseover=\"this.style.opacity=1.0\" onmouseout=\"this.style.opacity=0.5\" alt=\"" + localizedStrings['PreviousMemo'] + "\"/></a>";
}

function getToolMenu() {
	return "<a href=\"JavaScript:tool();\">" +
		"<img src=\"images/icon_tool.png\" id=\"icon_tool1\" onmouseover=\"this.style.opacity=0.5;\" onmouseout=\"this.style.opacity=0.25;\" alt=\"" + localizedStrings['Tool'] + "\"/></a>";
}

function getToolMenu2() {
	return "<a href=\"JavaScript:tool2();\">" +
		"<img src=\"images/icon_arrow-left.png\" id=\"icon_tool2\" onmouseover=\"this.style.opacity=1.0;\" onmouseout=\"this.style.opacity=0.5;\" alt=\"" + localizedStrings['Tool'] + "\"/></a>";
}

function getNextMenu(enable_flag) {
	var str = "";
	if (enable_flag) {
		str += "<a href=\"JavaScript:next();\">";
	} else {
		str += "<span style=\"color: #808080\">";
	}
	str += localizedStrings['Next'];

	if (enable_flag) {
		str += "</a>";
	} else {
		str += "</span>";
	}
	return str;
}
function getPreviousMenu(enable_flag) {
	var str = "";
	if (enable_flag) {
		str += "<a href=\"JavaScript:previous();\">";
	} else {
		str += "<span style=\"color: #808080\">";
	}
	str += localizedStrings['Previous'];

	if (enable_flag) {
		str += "</a>";
	} else {
		str += "</span>";
	}
	return str;
}

function getSeparatorTag() {
	return " <img src=\"images/separator.gif\" style=\"vertical-align:text-bottom\"/> ";
}
function getSpaceTag() {
	return "&nbsp;&nbsp;";
}

//--------------------------------------------
// focus event on body
//--------------------------------------------
function onfocusOnBody(event) {
	if (isBrightOnFocus()) {
		setOpacityToBright();
	}
}
function onblurOnBody(event) {
	if (isBrightOnFocus()) {
		setOpacity(getOpacity());
	}
}

//--------------------------------------------
// key down event
//--------------------------------------------
var _focus_keyword = false;

function onfocusAtKeyword(event) {
	_focus_keyword = true;
}
function onblurAtKeyword(event) {
	_focus_keyword = false;
}

function keypressOnViewer(event) {

//	debug(event.keyCode + "," + event.shiftKey + "," + event.metaKey);
	if (_focus_keyword) {
		return;
	}
	var scroll_bar = null;

	if (!isMini()) {
		if (isModeViewer()) {
			scroll_bar = viewer_scroll_bar;
		} else if (isModeResult()) {
			scroll_bar = result_scroll_bar;
		}

		if (scroll_bar != null &&
			// arrow keys (up & down)
			(event.keyCode == 63232 || event.keyCode == 63233)) {
			if (event.keyCode == 63232) {
				scroll_bar.lineUp();
				scroll_bar.lineUp();
			} else {
				scroll_bar.lineDown();
				scroll_bar.lineDown();
			}
			event.stopPropagation();
		}
	}
}

function keydownOnViewer(event) {
	if (_focus_keyword || isShowBack()) {
		return;
	}
//	debug(event.keyCode + "," + event.shiftKey + "," + event.metaKey);

	var isWorked = false;
	var scroll_bar = null;

	if (!isMini() && !isDialog()) {
		if (isModeViewer()) {
			scroll_bar = viewer_scroll_bar;
		} else if (isModeResult()) {
			scroll_bar = result_scroll_bar;
		}

		if (event.keyCode == 8) {
			// delete key
			if (isModeViewer()) {
				changeModeList(isListRecent()|isListDefault());
				isWorked = true;
			}

		} else if (event.keyCode == 13) {
			// delete key
			if (!event.shiftKey && isModeViewer()) {
				edit();
				event.stopPropagation();
			} else if (event.shiftKey && (isModeViewer() || isModeResult())) {
				newmemo();
				isWorked = true;
			}

		} else if (scroll_bar != null && event.keyCode == 32) {
			// space key (with shift key)
			if (event.shiftKey) {
				scroll_bar.pageUpOnce();
			} else {
				scroll_bar.pageDownOnce();
			}
			isWorked = true;

		} else if (scroll_bar != null &&
			// pageup & pagedown
			(event.keyCode == 33 || event.keyCode == 34)) {
			if (event.keyCode == 33) {
				scroll_bar.pageUpOnce();
			} else {
				scroll_bar.pageDownOnce();
			}
			isWorked = true;


		} else if (event.keyCode == 27) {
			// esc key
			if (isModeEditor()) {
				if (!isEmptyEditor()) {
					changeEditor();
				}
				endEdit();
				isWorked = true;
			}

		} else if (scroll_bar != null &&
			// arrow keys (up & down)
			(event.keyCode == 38 || event.keyCode == 40)) {
			if (event.keyCode == 38) {
				scroll_bar.lineUp();
			} else {
				scroll_bar.lineDown();
			}
			isWorked = true;

		} else if (event.keyCode == 37 || event.keyCode == 39) {
			// arrow keys (left & right)
			if (isModeViewer()) {
				if (event.keyCode == 37) {
					previousMemo();
				} else {
					nextMemo();
				}
				isWorked = true;

			} else if (isModeResult() && !isListSearch()) {
				if (event.keyCode == 37) {
					previous();
				} else {
					next();
				}
				isWorked = true;
			}
		} else if (event.metaKey && event.keyCode == 90) {
			// cmd+Z
			if (isModeEditor()) {
				if (event.shiftKey) {
					MemoStore.redo();
				} else {
					MemoStore.undo();
				}
				isWorked = true;
			}
		}
	}

	if (event.metaKey && !isModeEditor() && event.keyCode == 77) {
		// cmd+M
		if (isMini()) {
			hideMini();
		} else {
			showMini();
		}
		isWorked = true;
	}

	if (isWorked) {
		event.stopPropagation();
		event.preventDefault();
	}
}

//--------------------------------------------
// Utilities
//--------------------------------------------
var params = new Array();
var empty = "";
function getLocalizedString(key) {
	try {
		var str = localizedStrings[key];
		if (str === undefined) {
			str = key;
		}
		return str;
	} catch(ex) { }
	return key;
}

function text2html(str) {
	str = str.replace(/</g, "&lt;");
	str = str.replace(/>/g, "&gt;");
	str = str.replace(/\n/g, "<br/>");
	str = str.replace(/  /g, "&nbsp;&nbsp;");
	str = str.replace(/   /g, "&nbsp;&nbsp;&nbsp;");
	str = str.replace(/     /g, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
	str = str.replace(/       /g, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
	str = str.replace(/\t/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
	return str;
}
function decorateHitword(str, keyword) {
	keyword2 = keyword.replace(/\//g, '\\\/');
	keyword2 = keyword2.replace(/\\/g, '\\\\');
	var regx = eval("/(" + keyword2 + ")/ig");
	return str.replace(regx,
		"<span class=\"hitword\">$1</span>");
}
function decorateTitle(str) {
	return "<span class=\"title\">" + str +"</span>";
}
function clearEditor() {
	document.getElementById("editor").value = "";
}

function offFocusKeywordBox() {
//	document.getElementById("keyword").disable = true;
}

function convertToAppropriateCharacter(str) {
	return str.replace(/\xa0/g, " "); 
}

function convertURLToLinkTag(str) {
	return str.replace(/(https?:\/\/[#%&+,-.\/0-9;=?@A-Z_a-z~:]+)/g,
		"<a href=\"JavaScript:widget.openURL('$1');\">$1</a>");
}

// handling 'file://' scheme
function convertURLToLinkTag2(str) {

	var matches = str.match(/file:\/\/[^<]+/g);
	var appeared = new Array();

	for (idx in matches) {
		var url = matches[idx];
		
		if (!appeared[url]) {
			var url2 = url.replace(/^(file:\/\/)([^\/])/, "$1/$2");
			var url2 = url2.replace(/\/localhost\//, "//");
			var url3 = url2.replace(/^file:.+\/([^\/]+)\/?$/, "$1");
			var path = url2.replace(/^file:\/\/(.+)/, "$1");
			var icon = MemoStore.iconPathForFile(path);
			var icon_img = "";
			if (icon) {
				icon_img = "<img src=\"" + icon + "\" class=\"inline_icon\">";
			}
			var regex = new RegExp(url.replace(
				/([\^+*?${}|,\[\]\\\/()])/g, "\\$1"), "g");
			str = str.replace(
				regex,
				icon_img +
				"<a href=\"JavaScript:widget.openURL('" +
				url2.replace(/(['])/g, "\\$1") + "');\">" +
				url3 + "</a>");
			appeared[url] = "1";
		}
	}
	return str;
}

// img tag
function _getImageAttribute() {
	return " " +
		" onclick=\"mouseoverOnImage(event, this);\"" +
		" class=\"inline_image\"" +
		" ondragstart=\"ondragstartAtImage(event,this);\"" +
		" ondragover=\"ondragAtImage(event,this);\"" +
		" ondragend=\"ondragendAtImage(event,this);\"" +
		" onload=\"onLoadImage(this);\"";
}

// handling @capture
function convertURLToLinkTag3(str) {
	var capturePath = MemoStore.getCapturePath();
	var attr = _getImageAttribute();
	return str.replace(/@capture{([^}]+)}/g,
		"<img src=\"file:///" + capturePath + "/$1\"" + attr + "/>");
}

// mail link
function convertURLToLinkTag4(str) {
	return str.replace(
		/([^@\(\)<>,;:\\"\[\] ]+@[^@\(\)<>,;:\\"\[\]& ]+)/g,
		"<a href=\"JavaScript:widget.openURL('mailto:$1');\">$1</a>");
}


function convertURLToIMGTag(str) {
	var attr = _getImageAttribute();
	return str.replace(/(<a href="JavaScript:widget.openURL\('http[^>]+>)([^<]+\.(jpg|jpeg|gif|png|tiff|icns))(<\/a>)/ig,
		"<img src=\"$2\"" + attr + "/>");
}

function addIconOnAnchor(str) {
	return str.replace(/(<a href="JavaScript:widget.openURL\('http[^>]+>)([^<]+)(<\/a>)/ig,
		"<img src=\"images/icon_link.png\"" +
		" class=\"icon_link\"" +
		" onmouseover=\"this.style.opacity=1.0;\"" +
		" onmouseout=\"this.style.opacity=0.5;\"" +
		" ondragstart=\"ondragstartAtLink(event, '$2');\"" +
		" onclick=\"JavaScript:widget.openURL('$2');\"/> $1$2$3");
}


function removeDecorationInAHrefTag(str) {
	return str.replace(/(<a href="[^<]*)<span class="hitword">([^<]+)<\/span>([^<]*">)/g, "$1$2$3");
}
function removeDecorationInImgTag(str) {
	return str.replace(/(<img src="[^<]*)<span class="hitword">([^<]+)<\/span>/g, "$1$2");
}

// yyyy-mm-dd hh:mi:ss => mm/dd
function transDate1(str) {
	return str.replace(/....-(..)-(..) ..:..:../, "$1/$2");
}

// yyyy-mm-dd hh:mi:ss => mi:dd
function transDate2(str) {
	return str.replace(/....-..-.. (..:..):../, "$1");
}

// yyyy-mm-dd hh:mi:ss => mm-dd hh:mi
function transDate3(str) {
	return str.replace(/....-(..-.. ..:..):../, "$1");
}

// <-- yyyymmdd-hhmmss
function pre0(n) {
	if (n < 10) {
		return "0" + n;
	} else {
		return "" + n;
	}
}
function getDateString() {
	date    = new Date();
	year    = date.getYear() + 1900;
	month   = pre0(date.getMonth() + 1);
	day     = pre0(date.getDate());
	hours   = pre0(date.getHours());
	minutes = pre0(date.getMinutes());
	seconds = pre0(date.getSeconds());
	return year + "-" + month + "-" + day + "_" +
		hours + "." + minutes + "." + seconds;
}

//--------------------------------------------
// preferences
//--------------------------------------------
function isCaptureCommandAvailable() {
	return isFileExists(BIN_SCREENCAPTURE);
}

function isCaptureWindowMode() {
	var mode = widget.preferenceForKey(PKEY_CAPTURE_MODE);
	if (mode == null) {
		capture_mode = 0;
	}
	return (mode == 1);
}
function isCaptureInsertDate() {
	var flag = widget.preferenceForKey(PKEY_CAPTURE_DATE);
	if (flag == null) {
		flag = 1;
	}
	return (flag == 1);
}
function getOpacity() {
	var opacity = widget.preferenceForKey(PKEY_OPACITY);
	if (opacity == null) {
		opacity = 1.0;
	}
	return opacity;
}
function isBrightOnFocus() {
	var flag = widget.preferenceForKey(PKEY_BRIGHT_ONFOCUS);
	if (flag == null) {
		flag = 0;
	}
	return (flag == 1);
}

function isEmptyEditor() {
	var editor = document.getElementById("editor");

	return (editor.value == null || editor.value == "");
}

function removeFileExtension(filename) {
	return filename.replace("." + MemoStore.getFileExtension(), "");
}
function addFileExtension(str) {
	return str + "." + MemoStore.getFileExtension();
}

//--------------------------------------------
// Image
//--------------------------------------------
var _IMG_LIST;
var _IMG_WIDTH;
function initImageList() {
	_IMG_LIST  = new Array();
	_IMG_WIDTH = new Array();
}
function resizeImages() {
	for (idx in _IMG_LIST) {
		resizeImage(_IMG_LIST[idx], _IMG_WIDTH[idx]);
	}
}
function resizeImage(img, width) {
	var frame_width = parseInt(
		document.defaultView.getComputedStyle(
			document.getElementById("viewer"), "").getPropertyValue("width"));
	if (frame_width < width) {
		img.style.width="100%";
	} else {
		img.style.width="normal";
	}
}

function onLoadImage(img) {
	_IMG_LIST.push(img);
	_IMG_WIDTH.push(img.width);
	resizeImage(img, img.width);
	redrawScrollBar();
}

function mouseoverOnImage(event, img) {
	var width = img.width - 4;
	var height = img.height - 4;
	var x = img.offsetLeft;
	var y = img.offsetTop;

	var image_mask = document.getElementById("image_mask");
	image_mask.style.top = y + "px";
	image_mask.style.left = x + "px";
	image_mask.style.width = width + "px";
	image_mask.style.height = height + "px";
	image_mask.style.visibility = 'visible';

	var caputured_image = (img.src.match(/^file:/) != null);

	var html = "";
	if (caputured_image) {
		html += "<img src=\"images/icon_delete.png\"" +
			" class=\"icon_img_tool\"" +
			" onmouseover=\"this.style.opacity=1.0;\"" +
			" onmouseout=\"this.style.opacity=0.5;\"" +
			" onclick=\"img_del(event, '" + img.src + "');\"/>" +
			"&nbsp;";
	}
	html += "<img src=\"images/icon_open.png\"" +
		" class=\"icon_img_tool\"" +
		" onmouseover=\"this.style.opacity=1.0;\"" +
		" onmouseout=\"this.style.opacity=0.5;\"" +
		" ondragstart=\"ondragstartAtLink(event, '" + img.src + "');\"" +
		" onclick=\"img_open(event, '" + img.src + "');\"/>";


	image_mask.innerHTML = html;

	event.stopPropagation();
	event.preventDefault();
}
function mouseoutOnImage() {
	var image_mask = document.getElementById("image_mask");
	if (image_mask) {
		image_mask.style.visibility = 'hidden';
	}
}

function getImageMaskTag() {
	return "<div id=\"image_mask\"" +
		"</div>";
}

function img_open(event, src) {
	event.stopPropagation();
	event.preventDefault();
	widget.openURL(src);
	mouseoutOnImage();
}
var _delete_image_src;
function img_del(event, src) {
	event.stopPropagation();
	event.preventDefault();
	_delete_image_src = src;
	openDialog(getLocalizedString('MSG_ASK_DELIMG'), deleteCaptureImage);
}
function deleteCaptureImage(isYes) {
	if (isYes && _delete_image_src != null) {
		var filename = _delete_image_src.replace(
			/file:\/\/\//, "");

		var imagename = getFilenameFromFullPath(filename);

		var editor = document.getElementById("editor");
		var regex = new RegExp("@capture{" + imagename + "}\n?", "g");
		var content = editor.value.replace(regex, "");
		deleteFile(filename);
		uuid = getCurrentUUID();
		MemoStore.update(content, uuid);
		display(getCurrentSeq(), uuid);
		mouseoutOnImage();
	}
	_delete_image_src = null;
}

function ondragstartAtImage(event, image) {
	event.dataTransfer.effectAllowed = 'all';
	event.dataTransfer.setData('dummy', 'dummy');
	MemoStore.writeImageToPasteboard(event.x, event.y);
}
function ondragAtImage(event, image) {
}
function ondragendAtImage(event, image) {
}
function ondragstartAtLink(event, url) {
	event.dataTransfer.effectAllowed = 'all';
	event.dataTransfer.setData('NSStringPboardType', url);
}

//--------------------------------------------
// -
//--------------------------------------------
function createNewFile()
{
	var fileList = MemoStore.getFileList();
	var extension = MemoStore.getFileExtension();
	var count = 1;
	var MAXCOUNT = 100;
	var newFilename = "";
	var isExists = false;

	while (true) {
		newFilename = getLocalizedString('FilenamePrefix') +
			" " + count + "." + extension;
		isExists = false;
		for (var i=0; i < fileList.length; i++) {
			if (fileList[i] == newFilename) {
				isExists = true;
			}
		}
		if (!isExists) {
			break;
		}
		count++;
		if (MAXCOUNT <= count) break;
	}
	if (!isExists) {
		var result = MemoStore.createNewFile(newFilename);
		if (result) {
			setCurrentFilename(newFilename);
			MemoStore.setMemo(newFilename);
			setupFileList();
		}
	}

}
function selectFile(filename) {
	var select_file = document.getElementById("select_file");
	var length = select_file.length;
	var i;
	for (i=0; i < length; i++) {
		if (select_file.options[i].value == filename) {
			break;
		}
	}
	if (i != length) {
		select_file.selectedIndex = i;
	}
}

function changeFile()
{
	var select_file = document.getElementById("select_file");
	var filename =
		select_file.childNodes[select_file.selectedIndex].value;

	if (filename) {
		changeMemoFile(filename);
	} else {
		// create new memo
		createNewFile();
		showBack();
		document.getElementById("text_filename").focus();
	}
}

//--------------------------------------------
// back side functions
//--------------------------------------------
function commitSetting()
{
	var select_file = document.getElementById("select_file");
	var filename = addFileExtension(
		document.getElementById("text_filename").value);

	var trash_flag = 
		document.getElementById("checkbox_trash").checked;

	var rename_result = true;
	var trash_result = true;
	var result;
	var error_message = "";

	// (1) do something to the current file
	if (trash_flag) {
		trash_result = moveToTrash(getCurrentFilename());
		setCurrentFilename(getGenericMemoFilename());
		MemoStore.setMemo(getCurrentFilename());
		if (!trash_result) {
			error_message = getLocalizedString('MSG_ERR_TRASH');
		}
	} else {
		if (getCurrentFilename() != filename) {
			if (filename.match(/\//)) {
				error_message = getLocalizedString('MSG_ERR_WRONGCHAR');
				rename_result = false;
			} else {
				rename_result = renameFile(getCurrentFilename(), filename);
				if (rename_result) {
					setCurrentFilename(filename);
					MemoStore.setMemo(getCurrentFilename());
				} else {
					error_message = getLocalizedString('MSG_ERR_DUPNAME');
				}
			}
		}
	}

	result = rename_result && trash_result;

	if (result) {
		var previous_skin = getSkinName();
		var select_skin = document.getElementById("select_skin");
		var skin =
			select_skin.childNodes[select_skin.selectedIndex].value;
		if (skin != previous_skin) {
			setSkinName(skin);
			setupSkin(skin);
		}
	}

	if (result) {

		// (2) commit global preference
		widget.setPreferenceForKey(
			document.getElementById("text_mailto").value,PKEY_MAILTO);

		widget.setPreferenceForKey(
			document.getElementById("checkbox_capture").checked ? 1 : 0
			,PKEY_CAPTURE_MODE);

		widget.setPreferenceForKey(
			document.getElementById("checkbox_capture_date").checked ? 1 : 0
			,PKEY_CAPTURE_DATE);

		var select_opacity = document.getElementById("select_opacity");
		var opacity =
			select_opacity.childNodes[select_opacity.selectedIndex].value;
		widget.setPreferenceForKey(opacity, PKEY_OPACITY);
		widget.setPreferenceForKey(
			document.getElementById("checkbox_bright").checked ? 1 : 0
			,PKEY_BRIGHT_ONFOCUS);


		clearBackMessage();
		showFront();
	} else {
		setBackMessage(error_message);
	}
}
function revertSetting()
{
	clearBackMessage();
	showFront();
}

function clearBackMessage() {
	document.getElementById("back_message").innerHTML = "";
}
function setBackMessage(str) {
	document.getElementById("back_message").innerHTML =
		"<img src=\"images/icon_arrow-right2.gif\" class=\"icon_image\"/>&nbsp;" + str;
}

//--------------------------------------------
// dialog
//--------------------------------------------
var _dialog_flag;
var _post_function;
function openDialog(msg, postfunc) {
	_dialog_flag = true;
	document.getElementById("dialog_frame").style.display = 'block';
	document.getElementById("dialog_shadow").style.display = 'block';

	document.getElementById("dialog").innerHTML =
		msg + "<br/>" +
		"<a href=\"JavaScript:dialog_no();\">" +
			getLocalizedString("No") + "</a>" +
		getSeparatorTag() +
		"<a href=\"JavaScript:dialog_yes();\">" +
			getLocalizedString("Yes") + "</a>";
	_post_function = postfunc;
}
function closeDialog() {
	_dialog_flag = false;
	document.getElementById("dialog_frame").style.display = 'none';
	document.getElementById("dialog_shadow").style.display = 'none';
}
function dialog_yes(postfunc) {
	_post_function(true);
	closeDialog();
}
function dialog_no(postfunc) {
	_post_function(false);
	closeDialog();
}
function isDialog() {
	return _dialog_flag;
}

//--------------------------------------------
// Submenu slide animation
//--------------------------------------------
var _slide_timer = null;
var _slide_counter;
var _slide_start_right = 0;
var _slide_callback_function = null;
var _slide_width;
var SLIDE_ANIMATION_RES = 20;
var SLIDE_TIME = 250;
var SLIDE_MAX_WIDTH = 120;

function startSlide(callback, right_flag) {
	if (_slide_timer == null) {
		_slide_counter = 0;
		var slide_handler;
		if (right_flag) {
			slide_handler = slide_right1;
		} else {
			slide_handler = slide_left1;
		}

		_slider_timer =
			setInterval(slide_handler, SLIDE_TIME/SLIDE_ANIMATION_RES);
		_slide_callback_function = callback;
		var submenu = document.getElementById("submenu");
		_slide_width = SLIDE_MAX_WIDTH;
	}
}

function slide(right_flag) {
	var submenu = document.getElementById("submenu");
	var right = 0;
	_slide_counter++;
	var sign;
	if (right_flag) {
		sign = 1;
	} else {
		sign = -1;
	}

	right = _slide_start_right
		- (_slide_width / SLIDE_ANIMATION_RES) * _slide_counter * sign;

	submenu.style.right = right + "px";

	if (_slide_counter >= SLIDE_ANIMATION_RES) {
		clearInterval(_slider_timer);
		_slide_counter = 0;
		var slide_handler;
		if (right_flag) {
			slide_handler = slide_right2;
		} else {
			slide_handler = slide_left2;
		}
		_slider_timer =
			setInterval(slide_handler, SLIDE_TIME/SLIDE_ANIMATION_RES);
		_slide_width = parseInt(
			document.defaultView.getComputedStyle(
				submenu).getPropertyValue("width"));
		if (_slide_width > SLIDE_MAX_WIDTH) {
			_slide_width = SLIDE_MAX_WIDTH;
		}
		_slide_callback_function();
	}
}
function slide2(right_flag) {
	var submenu = document.getElementById("submenu");
	var right = 0;
	var sign;
	if (right_flag) {
		sign = 1;
	} else {
		sign = -1;
	}
	_slide_counter++;

	right = sign * (_slide_width - (_slide_width - _slide_start_right) / SLIDE_ANIMATION_RES * _slide_counter);

	submenu.style.right = right + "px";

	if (_slide_counter >= SLIDE_ANIMATION_RES) {
		clearInterval(_slider_timer);
		_slider_timer = null;
	}
}

function slide_right1() {
	slide(true);
}

function slide_right2() {
	slide2(true);
}

function slide_left1() {
	slide(false);
}
function slide_left2() {
	slide2(false);
}

//--------------------------------------------
// file management
//--------------------------------------------
function checkFilename(filename) {
	if (filename.match(/\.\./)) {
		return false;
	} else {
		return true;
	}
}
function deleteFile(filename) {
	var result;
	if (checkFilename(filename)) {
		result =
			(widget.system(BIN_RM + " " + filename, null).status == 0);
	} else {
		debug("bad path: " + filename);
		result = false;
	}
	return result;
}

function isFileExists(filename) {
	var cmd = widget.system(BIN_TEST + " -e " + filename, null);
	return (cmd.status == 0);
}

function renameFile(oldname, newname) {

	var result = false;
	if (checkFilename(oldname) && checkFilename(newname)) {
		result = MemoStore.renameFilename(oldname, newname);
	}
	return result;
}

function moveToTrash(filename) {
	var result = false;
	if (checkFilename(filename)) {
		result = MemoStore.moveToTrash(filename);
	}
	return result;
}
function getFilenameFromFullPath(fullpath) {
	return fullpath.replace(/^.+\/([^\/]+)$/, "$1");
}


//--------------------------------------------
// widget event handler
//--------------------------------------------
function setupWidget() {
	widget.onshow = widgetOnShow;
	widget.onremove = widgetOnRemove;
}

function widgetOnShow(event) {
}
function widgetOnRemove(event) {
	deleteStatus();
}


//--------------------------------------------
// save & load widget status
//--------------------------------------------
var WIDGET_STATUS_SEPARATOR = ',';

function getWidgetStatusKey() {
	return "WIDGET-" + widget.identifier;
}

// [0] width
// [1] height
// [2] filename
function saveStatus() {
	var width=window.outerWidth;
	var height=window.outerHeight;
	var filename=escape(getCurrentFilename());
	var mini = isMini() ? 1 : 0;

	var status = width + WIDGET_STATUS_SEPARATOR +
		height + WIDGET_STATUS_SEPARATOR +
		filename + WIDGET_STATUS_SEPARATOR +
		mini;
	widget.setPreferenceForKey(status, getWidgetStatusKey());
}

function loadStatus() {
	var status = widget.preferenceForKey(getWidgetStatusKey());
	var filename = null;
	if (status) {
		var attrs = status.split(WIDGET_STATUS_SEPARATOR);
		var width = attrs[0];
		var height = attrs[1];
		filename = unescape(attrs[2]);
		var mini = attrs[3];
		if (!MemoStore.isExistsFile(filename)) {
			filename = null;
		}
	}

	if (!filename) {
		filename = getGenericMemoFilename();
	}

	changeMemoFile(filename);
	resizeWidget(width, height);

	if (mini == 1) {
		showMini(true);
	}

}

function deleteStatus() {
	widget.setPreferenceForKey(null, getWidgetStatusKey());
}

//--------------------------------------------
// current filename management
//--------------------------------------------
var _current_filename;
function getCurrentFilename() {
	return _current_filename;
}
function setCurrentFilename(filename) {
	_current_filename = filename;
}

//--------------------------------------------
// memo management
//--------------------------------------------
function changeMemoFile(filename) {
	MemoStore.setMemo(filename);
	setCurrentFilename(filename);
	setMemoTitle(removeFileExtension(getCurrentFilename()));
	listOnMenu();
	selectFile(filename);
	saveStatus();

	setupSkin(getSkinName());
}
//--------------------------------------------
// skin management
//--------------------------------------------
function setupSkin(skinname) {
	if (!MemoStore.isExistsSkin(skinname)) {
		debug(skinname + " is not found");
		skinname = getSkinDefault();
	}
	var path = "skin/" + skinname + "/";

	var parts = new Array(
		"frame_top_left"   , "frame_top_mid"      , "frame_top_right",
		"frame_mid_left"   , "frame_mid_mid"      , "frame_mid_right",
		"frame_bottom_left", "frame_bottom_mid"   , "frame_bottom_right"
	);

	var id;
	for (index in parts) {
		id = parts[index];	
		document.getElementById(id).style.backgroundImage =
			"url(" + path + id + ".png)";
	}

	// icon
	document.getElementById("icon_mini").src = path + "icon_mini.png";
	document.getElementById("icon_normal").src = path + "icon_normal.png";

	// mini skin
	document.getElementById("mini").style.backgroundImage =
		"url(" + path + "mini.png)";
}

//--------------------------------------------
// opacity management
//--------------------------------------------
function setOpacity(opacity) {
	document.getElementById("front").style.opacity = opacity;
}
function getCurrentOpacity() {
	return document.getElementById("front").style.opacity;
}
function setOpacityToBright() {
	document.getElementById("front").style.opacity = 1.0;
}

