//--------------------------------------------
// 2005/11/30 Created by Hiroshi Hashiguchi
// Copyright (c) 2005
//--------------------------------------------
//------------------------------------
// debug
//------------------------------------
function DEBUG(str) {
	document.getElementById("debug").innerHTML += str + "<br/>";
}

//---------------------------------------------------------------------
// Global variables & constants
//---------------------------------------------------------------------
var SCROLL_SLIDER_MIN_HEIGHT = 16;
var PAGE_UPDOWN_INTERVAL     = 250; //[ms]

var DIRECTION_PAGE_UP        = 0;
var DIRECTION_PAGE_DOWN      = 1;

var SMOOTH_SCROLL_UNIT       = 10;

var SCROLL_SPEED_NORMAL      = 0;
var SCROLL_SPEED_HIGH        = 1;
var SCROLL_SPEED_LOW         = 2;

var MOUSE_WHEEL_RESOLUTION  = 120*1.5;

var SLIDER_OFFSET_Y          = 1;

//---------------------------------------------------------------------
// scroll list
//---------------------------------------------------------------------
var _MANAGER_LIST = new Array();

//---------------------------------------------------------------------
// scroll management class
//---------------------------------------------------------------------
function ScrollManager(
	document_frame  ,
	document_content,
	scroll_bar      ,
	scroll_slider   ,
	scroll_track)
{

	//-------------
	// properties
	//-------------
	this.frame             = document_frame;
	this.content           = document_content;
	this.bar               = scroll_bar;
	this.slider            = scroll_slider;
	this.track             = scroll_track ;
	this.slider_max_y      = 0;
	this.document_margin   = 0;
	this.mouse_down_y      = 0;
	this.slider_start_y    = 0;
	this.slider_min_height = SCROLL_SLIDER_MIN_HEIGHT;

	this.bar_offset_top    = getAbsoluteTop(this.bar);

	this.page_updown_timer = null;
	this.tracking_flag     = false;
	this.hidden            = true;

	this.page_updown_speed = SCROLL_SPEED_NORMAL;

	//--------------------------------------
	// save this for slider["id"] as key
	// NOTE: this var is used in events,
	//       because in event 'this' measns
	//       an object occurring the event.
	//--------------------------------------
	_MANAGER_LIST[this.slider["id"]] = this;
	_MANAGER_LIST[this.track["id"]] = this;

	//------------------------------------
	// methods
	//------------------------------------
	this.hideScrollBar = function() {
		this.track.style.visibility = "hidden";
		this.slider.style.visibility = "hidden";
		this.hidden = true;
	}

	this.showScrollBar = function () {
		this.track.style.visibility = "visible";
		this.slider.style.visibility = "visible";
		this.hidden = false;
	}

	this.isHidden = function() {
		return this.hidden;
	}

	this.redraw = function() {
		var content_height = parseInt(
			document.defaultView.getComputedStyle(
				this.content, "").getPropertyValue("height"));

		var frame_height = parseInt(
			document.defaultView.getComputedStyle(
				this.frame, "").getPropertyValue("height"));

		var bar_height = parseInt(
			document.defaultView.getComputedStyle(
				this.bar, "").getPropertyValue("height"));

		var rate_frame_content = frame_height / content_height;

		if (rate_frame_content < 1) {

			var slider_height = Math.max(
					Math.round(rate_frame_content * bar_height),
					this.slider_min_height);

			this.slider.style.height = slider_height;
			this.slider_max_y = bar_height - slider_height;
			this.document_margin = content_height - frame_height;

			var content_top = parseInt(this.content.style.top);
			var rest_space = this.document_margin + content_top;

			if (rest_space < 0) {
				var new_content_top = -rest_space + content_top;
				if (new_content_top > 0) {
					new_content_top = 0;
				}
				this.content.style.top = new_content_top + "px";

				var new_slider_top = (bar_height - slider_height) *
					(-new_content_top / this.document_margin);
				this.slider.style.top = new_slider_top;
			}

			this.showScrollBar();

		} else {

			this.content.style.top = "0px";
			this.slider_max_y = 0;
			this.hideScrollBar();
		}
	}

	this.scroll = function(new_y) {

		// move slider to new position
		if (new_y < SLIDER_OFFSET_Y) {
			new_y = SLIDER_OFFSET_Y;
		} else if (this.slider_max_y < new_y) {
			new_y = this.slider_max_y;
		}
		this.slider.style.top = new_y;

		// scroll content
		this.content.style.top =
			-this.document_margin * (new_y / this.slider_max_y);
	}

	this.wheelScroll = function(event) {
		var dy  = event.wheelDelta / MOUSE_WHEEL_RESOLUTION;
		var top = parseInt(
			document.defaultView.getComputedStyle(
				this.slider, "").getPropertyValue("top"));
		var ny = - dy + top;
		this.scroll(ny);
	}

	this.movePage = function(direction, smooth_scroll) {

		if (!this.tracking_flag || this.isHidden()) {
			return;
		}
		var frame_height = parseInt(
			document.defaultView.getComputedStyle(
				this.frame, "").getPropertyValue("height"));
		var content_top =parseInt(
			document.defaultView.getComputedStyle(
				this.content, "").getPropertyValue("top"));

		var new_content_top;

		var scroll_unit;
		if (smooth_scroll) {
			scroll_unit = SMOOTH_SCROLL_UNIT;
		} else {
			scroll_unit = frame_height;
		}

		if (direction == DIRECTION_PAGE_DOWN) {
			new_content_top = content_top - scroll_unit;
			if ((-new_content_top) > this.document_margin) {
				new_content_top = -this.document_margin;
				this.stopPageUpDownTimer();
			}
		} else {
			new_content_top = content_top + scroll_unit;
			if (new_content_top > 0) {
				new_content_top = 0;
				this.stopPageUpDownTimer();
			}
		}

		this.content.style.top = new_content_top;

		this.slider.style.top =
			this.slider_max_y * (-new_content_top/this.document_margin);

	}

	this.pageDown = function() {
		this.movePage(DIRECTION_PAGE_DOWN);
	}
	this.pageUp = function() {
		this.movePage(DIRECTION_PAGE_UP);
	}
	this.pageDownOnce = function() {
		this.tracking_flag = true;
		this.movePage(DIRECTION_PAGE_DOWN);
		this.tracking_flag = false;
	}
	this.pageUpOnce = function() {
		this.tracking_flag = true;
		this.movePage(DIRECTION_PAGE_UP);
		this.tracking_flag = false;
	}
	this.lineDown = function() {
		this.tracking_flag = true;
		this.movePage(DIRECTION_PAGE_DOWN, true);
		this.tracking_flag = false;
	}
	this.lineUp = function() {
		this.tracking_flag = true;
		this.movePage(DIRECTION_PAGE_UP, true);
		this.tracking_flag = false;
	}

	this.startPageUpDownTimer = function(direction) {
		var interval = PAGE_UPDOWN_INTERVAL;

		if (this.page_updown_speed == SCROLL_SPEED_LOW) {
			// low
			interval = PAGE_UPDOWN_INTERVAL * 3;
		} else if (this.page_updown_speed == SCROLL_SPEED_HIGH) {
			// high
			interval = Math.max(PAGE_UPDOWN_INTERVAL / 15, 10);
		}
		if (direction == DIRECTION_PAGE_UP) {
			this.page_updown_timer = setInterval(
				'_MANAGER_LIST["CURRENT"].pageUp();', interval);
		} else {
			this.page_updown_timer = setInterval(
				'_MANAGER_LIST["CURRENT"].pageDown();', interval);
		}
	}

	this.stopPageUpDownTimer = function() {
		clearInterval(this.page_updown_timer);
	}

	this.resetPosition = function() {
		this.content.style.top = 0;
		this.slider.style.top = SLIDER_OFFSET_Y;
	}

	//------------------------------------
	// slider event
	//------------------------------------
	this.mouseMoveSlider = function(event) {
		var manager = _MANAGER_LIST["CURRENT"];
		var dy = getEventY(event) - manager.mouse_down_y;
		var ny = manager.slider_start_y + dy;
		manager.scroll(ny);
	}

	this.mouseUpSlider = function(event) {
		var manager = _MANAGER_LIST["CURRENT"];

		document.removeEventListener(
			"mousemove", manager.mouseMoveSlider, true);
		document.removeEventListener(
			"mouseup", manager.mouseUpSlider, true);

		_MANAGER_LIST["CURRENT"] = null;
	}

	this.mouseDownSlider = function(event) {
		var manager = _MANAGER_LIST[this["id"]];
		_MANAGER_LIST["CURRENT"] = manager;

		document.addEventListener(
			"mousemove", manager.mouseMoveSlider, true);
		document.addEventListener(
			"mouseup", manager.mouseUpSlider, true);


		manager.mouse_down_y = getEventY(event);
		manager.slider_start_y = parseInt(
			document.defaultView.getComputedStyle(
				manager.slider, "").getPropertyValue("top"));
	}

	// add slider event
	this.slider.addEventListener(
		"mousedown", this.mouseDownSlider, true);

	//------------------------------------
	// Track event
	//------------------------------------
	this.mouseMoveTrack = function(event) {
		// none
	}

	this.mouseOverTrack = function(event) {
		var manager = _MANAGER_LIST["CURRENT"];
		manager.tracking_flag = true;
	}

	this.mouseOutTrack = function(event) {
		var manager = _MANAGER_LIST["CURRENT"];
		manager.tracking_flag = false;
	}

	this.mouseUpTrack = function(event) {
		var manager = _MANAGER_LIST["CURRENT"];

		manager.tracking_flag = false;
		manager.stopPageUpDownTimer();

		manager.track.removeEventListener(
			"mousemove", manager.mouseMoveTrack, false);
		manager.track.removeEventListener(
			"mouseover", manager.mouseOverTrack, false);
		manager.track.removeEventListener(
			"mouseout" , manager.mouseOutTrack , false);
		document.removeEventListener(
			"mouseup"  , manager.mouseUpTrack  , false);

		_MANAGER_LIST["CURRENT"] = null;
	}

	this.mouseDownTrack = function(event) {

		var manager = _MANAGER_LIST[this["id"]];
		_MANAGER_LIST["CURRENT"] = manager;

		this.addEventListener(
			"mousemove", manager.mouseMoveTrack, false);
		this.addEventListener(
			"mouseover", manager.mouseOverTrack, false);
		this.addEventListener(
			"mouseout" , manager.mouseOutTrack , false);
		document.addEventListener(
			"mouseup"  , manager.mouseUpTrack  , false);

		var slider_top = parseInt(
			document.defaultView.getComputedStyle(
				manager.slider, "").getPropertyValue("top")) +
				manager.bar_offset_top;

		manager.tracking_flag = true;

		if (getEventY(event) < slider_top) {
			manager.pageUp();
			manager.startPageUpDownTimer(DIRECTION_PAGE_UP);
		} else {
			manager.pageDown();
			manager.startPageUpDownTimer(DIRECTION_PAGE_DOWN);
		}
	}

	// add slider event
	this.track.addEventListener(
		"mousedown", this.mouseDownTrack, true);
}

//---------------------------------------------------------------------
// Utilities
//---------------------------------------------------------------------
function getEventY(event) {
	if (event.y === undefined) {
		// for firefox
		return event.pageY;
	} else {
		// for safari
		return event.y;
	}
}

function getAbsoluteTop(element) {

	var top;
	var total = 0;
	var i=0;
	while (element) {
		top = parseInt(
			document.defaultView.getComputedStyle(
				element, "").getPropertyValue("top"));

		if (isNaN(top)) {
			break;
		}
		total += top;
		element = element.parentNode;
	}

	return total;
}


