// Create the SNWEBVIEW object as our namespace
var SNWEBVIEW = (function () {

	// PRIVATES
	// the div that we're assuming contains our divs
	var skyDivID = "";
	
	// used in imageMap mode to switch between mapped image and
	// 	plain image (to improve browser performance during resize or gaze shift)
	var skyDivImageAndMap = null;
	var skyDivJustImage = null;

	var kMaxFOV = 90.0; // degrees
	var kMinFOV = 0.05; // degrees
	
	var kDefaultAdvertFrequency = 9; //how many times should the image load before advert is triggered

	// get this so that we can avoid an unnecessary (and dangerous) switch when the image calls onload
	var kPlaceHolderImageURL = 'http://a52.g.akamaitech.net/f/52/827/1d/www.space.com/snserver/googlegadget/images/placeholder.png';
	(new Image()).src = kPlaceHolderImageURL; // preload the (1x1!) image
	var kUIElemFadeTime = 1.0; // seconds
	var kUIElemMaxFade = 1.0; // opacity between 0 .. 1
	var kUIElemMinFade = 0.5; // opacity between 0 .. 1
	
	// housekeeping
	var gIgnoreClicks = false;
	
	var gImageRequestsSent = 0;
	var gAdsShown = 0;
	
	// Mouse wheel stuff
	var mouseWheelTempFOV = 90.0;
	var mouseWheelTimeout = null;
	
	//----------------------------------------------------------------------
	//	Class:	Watermark
	//
	//	Purpose:	Construct the Watermark object for inclusion in the SNWebView
	//
	//	Date			Initials	Version		Comments
	//  ----------	---------	----------	---------------------------
	//	2007/11/12		DWW			2.0.0		New
	//
	//----------------------------------------------------------------------
	var Watermark = Class.create(
	{
		initialize:  function(inParams)
		{
			var imgSrc		= inParams.imgSrc || 'http://www.space.com/snserver/img/snlogo.png';
			var opacity		= (inParams.opacity === undefined ) ? '1.0' : inParams.opacity ;
			var clickable	= (inParams.clickable === undefined) ? 'true' : inParams.clickable ;
			var url			= inParams.url || 'http://www.space.com/nightsky/applets.html';
			
			var myImg = new Image();
			myImg.src = imgSrc;
	
			var bgPre				= "background: url(";
			var bgPost				= ")";
			var watermarkImg		= bgPre + imgSrc + bgPost + " no-repeat;";
			watermarkImg 			+= " opacity: " + opacity; 
	
			// do this differently for IE6!
			if (navigator.appVersion.indexOf("MSIE") != -1)
			{	
				var temp=navigator.appVersion.split("MSIE");
				var version=parseFloat(temp[1]);
				if (version < 7.0)
				{
					var ieAlphaPre		= "background: none; filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='";
					var ieAlphaPost		= "', sizingMethod='crop )";
						
					watermarkImg		= ieAlphaPre + imgSrc + ieAlphaPost;
				}
			}
					
			this.watermarkHTML = '<div id=watermark style="position: absolute; right: 0px; bottom: 0px; width: '+myImg.width+'px; height: '+myImg.height+'px; '+watermarkImg+'"></div>';
			if (clickable) {
				this.watermarkHTML = '<a href="'+url+'" target="_blank" style=" cursor:hand; cursor:pointer;">' + this.watermarkHTML + "</a>";
			}		
	
		},
		getHTML : function()
		{
			return this.watermarkHTML;
		}		
	});

	//----------------------------------------------------------------------
	//	Class:	Advert
	//
	//	Purpose:	Construct the Advert object for inclusion in SNWebView
	//
	//	Date			Initials	Version		Comments
	//  ----------	---------	----------	---------------------------
	//	2008/01/10		MK			2.0.1		New
	//
	//----------------------------------------------------------------------
	var Advert = Class.create(
	{
		initialize:  function(inParams)
		{
			if (inParams === undefined)
			{
				inParams = {};
			}
			
			this.adCfgFile			= inParams.adCfgFile || null;
			this.adSrc				= inParams.adSrc 	 || 'http://www.space.com/snserver/ads/gadgetAds.html';
			this.frequency			= inParams.frequency || kDefaultAdvertFrequency;
			this.startOnFrame		= inParams.startOnFrame || 0;
			this.maxShows			= inParams.maxShows || 0;
			this.beforeAdShow 		= inParams.beforeAdShow || null;
			this.afterAdHide 		= inParams.afterAdHide || null;
			this.showAdMilliseconds = inParams.showAdMilliseconds || 4000;
			
			this.adWidth			= inParams.width	 || '300px';
			this.adHeight			= inParams.height	 || '250px';
			this.bgCSS				= inParams.bgCSS	 || 'black';
			
			this.advertHTML = "";
			
			if (this.adCfgFile)
			{
				this.requestAdvertCfg();
			}
			else
			{			
				this.advertHTML = '<table id="advert" style="position: absolute; display: none; background: '+ this.bgCSS +'; overflow: hidden; top: 0px; left: 0px; width: 100%; height: 100%; z-index: 10;">' +
								  	'<tr>' +
										'<td>' +
											'<iframe id="adFrame" marginheight="0" marginwidth="0" frameborder="0" src="'+ this.adSrc + '" scrolling="no" ' +
												'style="display: block; margin-left: auto; margin-right: auto; height: '+ this.adHeight +'; width: '+ this.adWidth +';"></iframe>' +
										'</td>' +
									'</tr>' +
								 '</table>';
			}
		},
		
		//if calling this independently, make sure that
		//	refreshAdvert() is called
		requestAdvertCfg: function()
		{
			var boundSuccessCallback = this.setCfgFromJSONResponse.bind(this);
				
			new Ajax.Request (this.adCfgFile,
			{
				asynchronous: false,
				method: 'get',
				evalJSON: 'force',
				onSuccess: boundSuccessCallback,
				onFailure: function(e) {;}
			});	
		},
		
		setCfgFromJSONResponse : function(transport)
		{
			var advertCfg = transport.responseJSON;

			if (advertCfg === null) //something went horribly wrong
				return;
											
			if (advertCfg.adsEnabled)
			{
				this.adSrc = advertCfg.adSrc;			
				this.startOnFrame = advertCfg.startOnFrame;
				this.frequency = advertCfg.frequency || kDefaultAdvertFrequency; //frequency cannot be 0
				this.maxShows = advertCfg.maxShows;
				this.showAdMilliseconds = advertCfg.duration;
				this.adHeight = advertCfg.adHeight;
				this.adWidth = advertCfg.adWidth;
				
				//reset counters since now we have a new ad config
				gImageRequestsSent = 0;
				gAdsShown = 0
				
				this.advertHTML = '<table id="advert" style="position: absolute; display: none; background: '+ this.bgCSS +'; overflow: hidden; top: 0px; left: 0px; width: 100%; height: 100%; z-index: 10;">' +
								  	'<tr>' +
										'<td>' +
											'<iframe id="adFrame" marginheight="0" marginwidth="0" frameborder="0" src="'+ this.adSrc + '" scrolling="no" ' +
												'style="display: block; margin-left: auto; margin-right: auto; height: '+ this.adHeight +'; width: '+ this.adWidth +';"></iframe>' +
										'</td>' +
									'</tr>' +
								 '</table>';
			}
		},
		
		getHTML : function()
		{
			return this.advertHTML;
		},
		
		getFrequency : function()
		{
			return this.frequency;
		},
		
		getStartOnFrame : function()
		{
			return this.startOnFrame;
		},
		
		getMaxShows : function()
		{
			return this.maxShows;
		},
		
		showAdvert : function()
		{
			if (this.advertHTML)
			{
				if (this.beforeAdShow) {
					Effect.Appear($("advert"),{beforeStart:this.beforeAdShow});
				}
				else{
					Effect.Appear($("advert"));
				}
	
				if (SNWEB.trackAction) {
					SNWEB.trackAction( {action: SNWEB.consts.kTrack_adDisplayed, isError: false, data: this.adSrc });
				}
			
				setTimeout(this.hideAdvert.bind(this), this.showAdMilliseconds);
			}
			else{
				
			}
		},

		hideAdvert : function()
		{
			if (this.advertHTML)
			{
				if (this.afterAdHide) {
					Effect.Fade($("advert"),{afterFinish: this.afterAdHide});
				}
				else{
					Effect.Fade($("advert"));
				}
				setTimeout(this.refreshAdvert.bind(this), 1500); //switch ad after hiding so that it doesn't do the annoying swap on show
			
				
			}
		},
		refreshAdvert : function()
		{
			if (this.advertHTML)
			{
				var theWindow = $('adFrame').contentWindow;
				if (false === (!!theWindow))
				{
					theWindow = $('adFrame').contentDocument;
				}
				
				if (theWindow)
				{
					theWindow.location.replace(this.adSrc); 
				}
			}
		}
		
	});
	

	//----------------------------------------------------------------------
	//	Class:	SNWebViewClass
	//
	//	Purpose:	View class that creates and handles events for a SN web view object
	//
	//	Date			Initials	Version		Comments
	//  ----------	---------	----------	---------------------------
	//	2007/11/12		DWW			2.0.0		New
	//
	//----------------------------------------------------------------------
	var SNWebViewClass = Class.create(
	{
		initialize: function()
		{
			this.imageRefreshed = null;
			this.flowingTime = false;

			// Pan, zoom and cardinal button controls
			this.panning	= true;
			this.zooming	= true;
			this.cardinals	= true;
			
			this.minFOV = kMinFOV;
			
			// if out of focus opacity is diff from in focus opacity then we employ fade during focus switch
			this.controlsOutOfFocusOpacity = 1.0;
			this.controlsInFocusOpacity = 1.0;
			
			//show ads
			this.advert		= null;		

			// watermark
			this.watermark = null;
		
			this.imageMap = false;
			
			//tooltip specs
			this.tooltip = {
								backgroundColor: "#F4F4F4",
								color: "#000000",
								opacity: 0.7,
								width: 150
							};
											
			// a reference to the situation
			this.situation = null;

			// the zoom slider
			this.zoomSlider = null; // create this in the init if necessary

		},
		// -------------------------------------------------
		//	init
		// -------------------------------------------------
		init: function(inParams)
		{
			// set the vars for the other UI components
			if (inParams)
			{			
				this.panning	= (inParams.panning === undefined) ?	this.panning :	inParams.panning;
				this.zooming	= (inParams.zooming === undefined) ?	this.zooming :	inParams.zooming;
				this.cardinals	= (inParams.cardinals === undefined) ?	this.cardinals : inParams.cardinals;
				this.minFOV	= (inParams.minFOV == undefined) ? this.minFOV : inParams.minFOV;
				
				this.controlsOutOfFocusOpacity = (inParams.controlsOutOfFocusOpacity === undefined) ? this.controlsOutOfFocusOpacity : inParams.controlsOutOfFocusOpacity;
				this.controlsInFocusOpacity = (inParams.controlsInFocusOpacity === undefined) ? this.controlsInFocusOpacity : inParams.controlsInFocusOpacity;
				
				this.advert		= (inParams.advert === undefined) ? this.advert : new Advert(inParams.advert);				
				this.watermark	= (inParams.watermark === undefined) ?	this.watermark : new Watermark(inParams.watermark);				
				this.flowingTime= (inParams.flowingTime === undefined) ? this.flowingTime : inParams.flowingTime;
				this.situation	= inParams.situation || new SNWEB.SNSituation();
				
				this.imageMap	= (inParams.imageMap === undefined) ?	this.imageMap:	inParams.imageMap;
				
				if (this.imageMap && (inParams.tooltip !== undefined))
				{
					this.tooltip.backgroundColor = (inParams.tooltip.backgroundColor === undefined) ? this.tooltip.backgroundColor : inParams.tooltip.backgroundColor;
					this.tooltip.color = (inParams.tooltip.color === undefined) ? this.tooltip.color : inParams.tooltip.color;
					this.tooltip.opacity = (inParams.tooltip.opacity === undefined) ? this.tooltip.opacity : parseFloat(inParams.tooltip.opacity);
					this.tooltip.width = (inParams.tooltip.width === undefined) ? this.tooltip.width : parseInt(inParams.tooltip.width);
				}				
			}
	
			// in case we passed no params, we still want a situation
			if (this.situation === null) {
				this.situation = new SNWEB.SNSituation();
			}
	
			// setup the SNWEB object for our gadget.
			SNWEB.platform	= inParams.platform	|| SNWEB.consts.kUnknownPlatform;
			SNWEB.version	= inParams.version	|| "0.0.0";
		
			// inject the necessary HTML
			var i = 0;
			var theElemsArray = null;
			theElemsArray = SNWEB.utils.getElementsByClassName("snSkyView");
			for (i = 0; i < theElemsArray.length; i += 1)
			{
				skyDivID = theElemsArray[i].id;		
				this.injectWidgetHTML(theElemsArray[i]);
				break; // only do this for the first one for now!
			}
			
			// hide any UI elements that we don't want (note that it's important to hide these rather than not include their HTML so that we don't have layout issues)
			if ((this.panning === false) && ($("gazeControls"))) {
				$("gazeControls").hide();
			}
			
			if (this.zooming === false)
			{
				if ($("zoomSliderTrack"))
				{
					$("fovDisplay").hide();
					$("zoomInButton").hide();
					$("zoomSliderTrackTop").hide();
					$("zoomSliderTrack").hide();
					$("zoomSliderHandle").hide();
					$("zoomSliderTrackBottom").hide();
					$("zoomOutButton").hide();
				}
			} else {
				this.zoomSlider = new Control.Slider('zoomSliderHandle', 'zoomSliderTrack', {axis: 'vertical', range:$R(this.minFOV,kMaxFOV), sliderValue: this.situation.gaze.FOV}); //
			}
			
			if ((this.cardinals  === false) && ($("cardinalButtons"))) {
				$("cardinalButtons").hide();
			}
			
			// track that we have been initialized
			if (SNWEB.trackAction) {
				SNWEB.trackAction({action: SNWEB.consts.kTrack_Initialized});
			}
	
			// Attach the event handlers.
			this.attachEventHandlers();
			
			// update the FOV the first time manually
			this.updateFOVDisplay(this.situation.gaze.FOV);
			
			// Request an image
			this.refreshImage();
		},
		// -------------------------------------------------
		//	refreshImage
		// -------------------------------------------------
		refreshImage: function()
		{
			// before we request the new image, 
			if (this.flowingTime) {
				this.situation.dateTime = new Date();
			}
	
			// One final check to see if the horizon is ON and we're trying to view below the horizon
			if ((this.situation.options.horizon === true) && (this.situation.gaze.alt < 0)) {
				this.situation.gaze.alt = SNWEB.consts.kDefaultAltitude;
			}

			// Now let's strip the image map code and use plain image code in case this hasn't already been done
			this.removeImageMap();
			
			if (this.advert)
			{
		        if (
		        	(gImageRequestsSent >= this.advert.getStartOnFrame()) &&
		        	( (this.advert.getMaxShows() == 0) || (this.advert.getMaxShows() > gAdsShown) ) &&
		        	( ((gImageRequestsSent - this.advert.getStartOnFrame()) % this.advert.getFrequency()) == 0 )
		        )
        		{
        			var showMyAdvert = this.advert.showAdvert.bind(this.advert);			
					showMyAdvert();
					gAdsShown += 1;
				}
				gImageRequestsSent += 1;
			}
			
			this.requestImageForSituation();
		},
		// -------------------------------------------------
		//	imageLoaded
		// -------------------------------------------------
		imageLoaded: function(inEvt)
		{
			// event handling boilerplate
			var target = this || inEvt.element(); // FOR the IMG theElems only, we use the 'this' item to get at them first
			inEvt.stop();
						
			// don't switch if we've just loaded the placeholder image for the first time.
			if (target && target.src && (target.src != kPlaceHolderImageURL) && SNWEBVIEW) {
				SNWEBVIEW.switchImageDivs(); // HAVE to use SNWEBVIEW here because of stupid binding issues
			}
		},
//  needs test across all  browsers
		getControlEventTarget: function(inEvt){
			
			var target = null;
			if(gIgnoreClicks === false){  // mouse clicks or key press
				if(inEvt.type == "keypress"){
					var key=(inEvt.charCode)?inEvt.charCode: 
							((inEvt.keyCode)?inEvt.keyCode:
							((inEvt.which)?inEvt.which:0));
					if (key >= 63232 || key == Event.KEY_UP || key == Event.KEY_DOWN 
						|| key == Event.KEY_RIGHT || key == Event.KEY_LEFT ){  //Safari returns  different unicode keycodes 
						switch (key)					       //Event.KEY_* uses standard unicode keycodes (also ascii)
						{
							case 63232 : key = Event.KEY_UP; break;   //if safari keycode found convert to standard
							case 63233 : key = Event.KEY_DOWN; break; 
							case 63234 : key = Event.KEY_LEFT; break;
							case 63235 : key = Event.KEY_RIGHT; break;
						}
					}
					else{  //all other key return a string   
						key = String.fromCharCode(key);
					}
				
					target = key;  
				}
				else{ 
					target = inEvt.element().id;
					inEvt.stop();
				}
			}	
			return target;
		
		},
		onKeyEvent: function(inKey){
			var target = this.getControlEventTarget(inKey);
			switch (target)
			{
				case "N":
				case "n":
				case "S":
				case "s":
				case "E":
				case "e":
				case "W":
				case "w":
					this.faceDirection(inKey);
				break;
				case "d": case "D":
					this.situation.options.daylight = !this.situation.options.daylight;
					document.getElementById("daylightIconDiv").style.display=this.situation.options.daylight?"none":"block";
					setInfoButtonStyle();
					this.refreshImage();
				break;
				case Event.KEY_LEFT:
				case Event.KEY_RIGHT:
				case Event.KEY_UP:
				case Event.KEY_DOWN:
					this.incrementGaze(inKey);
				break;
				
				case "+":case "=":
				case "-":case "_":
					this.stepZoomHandler(inKey);
				break;
				default:
					return;
			}

			
		}, 
		// -------------------------------------------------
		//	incrementGaze
		// -------------------------------------------------
		incrementGaze: function(inEvt)
		{
			var target = this.getControlEventTarget(inEvt);
			if (target == null){
				return;
			}			
			var kFractionToMove = 1/6; // just move 1/6 of the current FOV
			var amountToMove = this.situation.gaze.FOV * kFractionToMove; 
			
			var pixelsToMove = $("skyDiv").clientHeight*kFractionToMove;
			var pixelsToMoveVert = 0;
			var pixelsToMoveHorz = 0;
		
			var kMaxAltitude = 89.9;
			var kMinAltitude = -89.9;
			
			var kAltJustAboveHorizon = 0.001;
		
			var kMaxAzimuth = 360.0;
			var kMinAzimuth = 0.0;
			
			if ((amountToMove > kMaxAzimuth) || (amountToMove > 2*kMaxAltitude))
			{
				alert("Error");
				return;
			}
		
			var currentAlt = this.situation.gaze.alt;
			var currentAz = this.situation.gaze.az;

			switch (target)
			{
				case "gazeUpButton": case Event.KEY_UP:
					if (currentAlt == kMaxAltitude) {
						return; //nowhere left to go
					}
					else if ((currentAlt + amountToMove) < kMaxAltitude) {
						this.situation.gaze.alt += amountToMove;
						pixelsToMoveVert = pixelsToMove;
					}
					else {
						this.situation.gaze.alt = kMaxAltitude;
					}
					break;
				case "gazeDownButton": case Event.KEY_DOWN:
					if (this.situation.options.horizon === true)
					{
						if (currentAlt == kAltJustAboveHorizon) {
							return; // bail out if were already at the minimum
						}
						else if ((currentAlt - amountToMove) < 0) {
							this.situation.gaze.alt = kAltJustAboveHorizon;
						}
						else {
							this.situation.gaze.alt -= amountToMove;
							pixelsToMoveVert -= pixelsToMove;
						}
					}
					else
					{
						if (currentAlt == kMinAltitude) {
							return; // nowhere left to go!
						}
						else if ((currentAlt - amountToMove) > kMinAltitude) {
							this.situation.gaze.alt -= amountToMove;
							pixelsToMoveVert -= pixelsToMove;
						}
						else {
							this.situation.gaze.alt = kMinAltitude;
						}
					}
					break;
				case "gazeLeftButton": case Event.KEY_LEFT:
					if ((currentAz - amountToMove) > kMinAzimuth) {
						this.situation.gaze.az -= amountToMove;
					}
					else {
						this.situation.gaze.az = (kMaxAzimuth + currentAz - amountToMove);
					}
		
					pixelsToMoveHorz = pixelsToMove;
					break;
				case "gazeRightButton": case Event.KEY_RIGHT:
					if ((currentAz + amountToMove) < kMaxAzimuth) {
						this.situation.gaze.az += amountToMove;
					}
					else {
						this.situation.gaze.az = (currentAz + amountToMove - kMaxAzimuth);
					}
			
					pixelsToMoveHorz -= pixelsToMove;
					break;
				default:
					return; 
			}									
					
			// make the request right away so that we'll be downloading the image while moving the old one
			// refresh image strips the imageMap
			this.refreshImage();
		
			if ((pixelsToMoveHorz !== 0) || (pixelsToMoveVert !== 0)) {
				(new Effect.Move(this.getVisibleSkyDiv(), {x: pixelsToMoveHorz, y: pixelsToMoveVert, mode: 'relative', duration: 0.5})); // afterFinish: refreshImage
			}
		},
		// -------------------------------------------------
		//	faceDirection
		// -------------------------------------------------
		faceDirection: function(inEvt)
		{
			var target = this.getControlEventTarget(inEvt);
			if (target == null){
				return;
			}						
			if (target == "showNorthButton" || target =="n" ||  target =="N") {
				this.situation.gaze.az = 0.0;
			}
			else if (target == "showEastButton" || target =="e" ||  target =="E")	{
				this.situation.gaze.az = 90.0;	
			}
			else if (target == "showSouthButton" || target =="s" ||  target =="S") {
				this.situation.gaze.az = 180.0;	
			}
			else if (target == "showWestButton" || target =="w" ||  target =="W") {
				this.situation.gaze.az = 270.0;
			}
			else {
				return;
			}
			
			var kDefaultAltitude = 15.0;
			this.situation.gaze.alt = kDefaultAltitude;
			
			if (this.zoomSlider) {
				this.zoomSlider.setValue(90); // always want to zoom out to 90 for these buttons
			}
			else {
				this.situation.gaze.FOV = 90;
				this.refreshImage();
			}
		},
		// -------------------------------------------------
		//	zoomSliderSlid
		// -------------------------------------------------
		zoomSliderSlid: function(inNewFOV)
		{
			this.scaleSkyImageForFOV(inNewFOV);
			this.updateFOVDisplay(Number(inNewFOV).toFixed(2));
		},
		// -------------------------------------------------
		//	zoomSliderChanged
		// -------------------------------------------------
		zoomSliderChanged: function(value)
		{
			this.situation.gaze.FOV = value;
			this.updateFOVDisplay(value);
			mouseWheelTempFOV = value;
			this.refreshImage();
		},
		// -------------------------------------------------
		//	attachEventHandlers
		// -------------------------------------------------
		attachEventHandlers: function()
		{
			function attachHandlerToAllElemsOfClass(inClassName, inEventName, inHandler)
			{
				var i = 0;
				var theElemsArray = null;
				theElemsArray = SNWEB.utils.getElementsByClassName(inClassName);
				for (i = 0; i < theElemsArray.length; i += 1)
				{
					Event.observe(theElemsArray[i], inEventName, inHandler);
				}
			}
	
			// Be sure that our callbacks are correctly bound to this object
			
			attachHandlerToAllElemsOfClass("skyImage", "load", this.imageLoaded); // DON'T bind the image loaded callbacks or you don't get the correct image element
			attachHandlerToAllElemsOfClass("gazeButton", "click", this.incrementGaze.bind(this));
			attachHandlerToAllElemsOfClass("zoomButton", "click", this.stepZoomHandler.bind(this));
			attachHandlerToAllElemsOfClass("cardinalButton", "click", this.faceDirection.bind(this));
		
			if (this.controlsOutOfFocusOpacity != this.controlsInFocusOpacity)
			{
				Event.observe("controls", "mouseover", (function(inEvt) { this.fadeUpUIElement($("controls")); }).bind(this));
				Event.observe("controls", "mouseout", (function(inEvt) { this.fadeDownUIElement($("controls")); }).bind(this));
				Event.observe("cardinalButtons", "mouseover", (function(inEvt) { this.fadeUpUIElement($("cardinalButtons")); }).bind(this));
				Event.observe("cardinalButtons", "mouseout", (function(inEvt) { this.fadeDownUIElement($("cardinalButtons")); }).bind(this));			
				
				this.fadeDownUIElement($("controls"));
				this.fadeDownUIElement($("cardinalButtons"));
			}

			if (this.zoomSlider)
			{
				this.zoomSlider.setEnabled();	
				this.zoomSlider.options.onChange = this.zoomSliderChanged.bind(this);
				this.zoomSlider.options.onSlide = this.zoomSliderSlid.bind(this);
				
				// only add mouse wheel zooming if we have the zoom slider
				Event.observe("skyDiv", 'mousewheel', this.mouseWheelMoved.bind(this));
				Event.observe("skyDiv", 'DOMMouseScroll', this.mouseWheelMoved.bind(this)); // DOMMouseScroll is for mozilla.
			}
		},		
		// -------------------------------------------------
		//	stepZoom
		// -------------------------------------------------
		stepZoom: function(zoomIn, zoomStep)
		{
			// ensure that zoomIn is Boolean
			zoomIn = !!zoomIn;
			
			var newFOV = this.situation.gaze.FOV;
	
			if (zoomIn)
			{
				newFOV 	-= zoomStep;
				if (newFOV < this.minFOV)
				{
					newFOV = this.minFOV;
				}
			}
			else
			{
				newFOV 	+= zoomStep;
				if (newFOV > kMaxFOV)
				{
					newFOV = kMaxFOV;
				}
			}
		
			var prevValue = this.situation.gaze.FOV;
			var scaling = (prevValue / newFOV * 100);
			
			//get rid of image map so that image scaling happens smoothly
			this.removeImageMap();
			var srctochange = this.getVisibleImgElement();
			if(srctochange){
				(new Effect.Scale(srctochange, scaling, {scaleFromCenter: 'true', duration: 1.0}));
			
				if (this.zoomSlider) {
					this.zoomSlider.setValue(newFOV);
				}
			
				else { 
					this.situation.gaze.FOV = newFOV;
					this.refreshImage();
				}
			}
		},
		// -------------------------------------------------
		//	stepZoomHandler
		// -------------------------------------------------
		stepZoomHandler: function(inEvt)
		{
		
			var target = this.getControlEventTarget(inEvt)
			if(target == null){
				return
			}
			var stepSize = this.getZoomStep(this.situation.gaze.FOV);	
			var zoomIn = ((target == "zoomInButton") || (target == "+")) || (target =="=");
			
			this.stepZoom(zoomIn, stepSize);		
		},
		// -------------------------------------------------
		//	mouseWheelMoved
		// -------------------------------------------------
		mouseWheelMoved: function(inEvt)
		{
			if (gIgnoreClicks === true)
			{
				return;
			}
	
			// event handling boilerplate
			inEvt = inEvt || window.event;
	
			inEvt.cancelBubble = true;
			if (inEvt.stopPropogation) {
				inEvt.stopPropogation();
			}
			
			var delta = 0;
					
			if (inEvt.wheelDelta)
			{
				delta = inEvt.wheelDelta/120;  /* IE/Opera. */			
			}
			else if (inEvt.detail)
			{
				/** Mozilla case. */
				/** In Mozilla, sign of delta is different than in IE.
				 * Also, delta is multiple of 3.
				 */
				delta = -inEvt.detail/3;
			}
			
			/** If delta is nonzero, handle it.
			* Basically, delta is now positive if wheel was scrolled up,
			* and negative, if wheel was scrolled down.
			*/
			if (delta)
			{
				if (mouseWheelTimeout !== null)
				{
					clearTimeout(mouseWheelTimeout);
					mouseWheelTimeout = null;
				}
	
				var fovCheck = mouseWheelTempFOV - delta*1.6;
				if (fovCheck > kMaxFOV) {
					mouseWheelTempFOV = kMaxFOV;
				} else if (fovCheck < this.minFOV){
					mouseWheelTempFOV = this.minFOV;
				}
				else {
					mouseWheelTempFOV = fovCheck;
				}
		
				this.zoomSliderSlid(mouseWheelTempFOV);
				mouseWheelTimeout = setTimeout(this.mouseWheelFinished.bind(this), 500, mouseWheelTempFOV);
			}
					
			/** PrinEvt default actions caused by mouse wheel.
			* That might be ugly, but we handle scrolls somehow
			* anyway, so don't bother here..
			*/
			if (inEvt.preventDefault)
			{
				inEvt.preventDefault();
			}
			
			inEvt.returnValue = false;
		},
		// -------------------------------------------------
		//	mouseWheelFinished
		// -------------------------------------------------
		mouseWheelFinished: function(inFinalFOV)
		{
			inFinalFOV = inFinalFOV | mouseWheelTempFOV; // need this since IE won't pass this var through setTimeout
			if (this.zoomSlider) {
				this.zoomSlider.setValue(inFinalFOV);
			}
			else {
				this.zoomSliderChanged(inFinalFOV);
			}
			
			mouseWheelTempFOV = this.situation.gaze.FOV;
		},
		// -------------------------------------------------
		//	getZoomStep
		// -------------------------------------------------
		getZoomStep: function(inCurrentFOV)
		{
			var zoomStep = 15;
			if (inCurrentFOV < 45)
			{
				zoomStep = 10;
				if (inCurrentFOV < 30)
				{
					zoomStep = 5;
					if (inCurrentFOV < 15)
					{
						zoomStep = 3;
						if (inCurrentFOV < 5)
						{
							zoomStep = 1;
							if (inCurrentFOV < 2)
							{
								zoomStep = 0.5;
								if (inCurrentFOV < 1)
								{
									zoomStep = 0.1;
								}
							}
						}
					}
				}
			}
			
			return zoomStep;
		},
		// -------------------------------------------------
		//	scaleSkyImageForFOV
		// -------------------------------------------------
		scaleSkyImageForFOV: function(inNewFOV)
		{
			var scaling = 1.0;
			if (this.situation.gaze.FOV) {
				scaling = (this.situation.gaze.FOV / inNewFOV);
			}
			else {
				return;
			}			
	
			//if image map hasn't already been stripped off, do it now so that image scaling goes smoothly
			this.removeImageMap();
			
			var skyDivDims = $("skyDiv").getDimensions();
			var origHeight = parseInt(skyDivDims.height, 10);
			var origWidth = parseInt(skyDivDims.width, 10);

			var imageSrcToChange = this.getVisibleImgElement();
			if(imageSrcToChange){
				imageSrcToChange.setStyle(
					{
						width: (origWidth * scaling) + "px",
						height: (origHeight * scaling) + "px",
						top: ((1 - scaling) * origHeight / 2.0) + "px",
						left: ((1 - scaling) * origWidth / 2.0) + "px"
					}
				);
			}
		},
		// -------------------------------------------------
		//	updateFOVDisplay
		// -------------------------------------------------
		updateFOVDisplay: function(inFOV)
		{
			var fovDisp = $("fovDisplay");
			if (fovDisp) {
				fovDisp.innerHTML = Number(inFOV).toFixed(2);
			}
		},
		// -------------------------------------------------
		//	getVisibleSkyDiv
		// -------------------------------------------------
		getVisibleSkyDiv: function()
		{
			if ($("skyDiv1").style.display == "none") {
				return $("skyDiv2");
			}
			else {
				return $("skyDiv1");
			}
		},
		// -------------------------------------------------
		//	requestImageForSituation
		// -------------------------------------------------
		requestImageForSituation: function()
		{
			gIgnoreClicks = true; // ignore further clicks until we get a response
	
			// Create the request from the situation
			var theWidth = $("skyDiv").clientWidth;
			var theHeight = $("skyDiv").clientHeight;
			var reqType;
			
			if (this.imageMap)
			{
				reqType = SNWEB.consts.kGenerateImageAndMapRequest;
			}
			else
			{
				reqType = SNWEB.consts.kGenerateImageRequest;
			}
			
			var requestURL = SNWEB.getRequestURL({	type:		 reqType,
													situ:		 this.situation,
													getFullHTML: !this.imageMap, //we expect JSON, not HTML
													width:		 theWidth,
													height:		 theHeight});
			
			// be sure to bind these two calls to this object so that their 'this' pointer is correct.
			var boundSuccessCallback;
			var boundFailureCallback = this.imageRequestFailure.bind(this);
			
			if (this.imageMap)
			{
				boundSuccessCallback = this.imageAndMapRequestSuccess.bind(this);
			}
			else
			{
				boundSuccessCallback = this.imageRequestSuccess.bind(this);
			}

			new Ajax.Request(requestURL,
			{ 
				method: 'get',
				onSuccess: boundSuccessCallback,
				onFailure: boundFailureCallback
			});
		},		
		//-------------------------------------------------
		//	imageRequestSuccess
		// -------------------------------------------------
		imageRequestSuccess: function(transport)
		{
			var newImageSrc = transport.responseText;
			SNWEB.trackAction({action: SNWEB.consts.kTrack_ImageRequest});
			
			var imageSrcToChange = this.getHiddenImgElement();
			imageSrcToChange.src = newImageSrc;				
			
			// call the image refreshed callback if the user has defined one
			if (this.imageRefreshed) {
				this.imageRefreshed(); 
			}
		},
		//-------------------------------------------------
		//	imageAndMapRequestSuccess
		// -------------------------------------------------
		imageAndMapRequestSuccess: function(transport)
		{
			var imageAndMapData = transport.responseJSON;
			var obj;
			var ddElem, aElem;			
			
			if (imageAndMapData === null) //something went horribly wrong
				return;
											
			skyDivImageAndMap = document.createElement("dl");
			skyDivImageAndMap.setAttribute("id", "skyMap");
			
			skyDivImageAndMap.style.position = "absolute";
			skyDivImageAndMap.style.margin = "0px";
			skyDivImageAndMap.style.padding = "0px";			
			skyDivImageAndMap.style.background = "url('"+imageAndMapData.imageInfo.url+"') no-repeat";
			skyDivImageAndMap.style.width = imageAndMapData.imageInfo.width + "px";
			skyDivImageAndMap.style.height = imageAndMapData.imageInfo.height + "px";
			skyDivImageAndMap.style.fontFamily = "'Lucida Grande', 'Lucida Sans Unicode', Geneva, Arial, Helvetica, sans-serif";
			skyDivImageAndMap.style.fontSize = ( 85 * this.tooltip.width / 150 ) + "%"; //assume that 85% font on 150px width of tooltip is optimal ratio
			
			for (var i=0, len=imageAndMapData.skyObject.length; i< len; ++i)
			{				
				obj = imageAndMapData.skyObject[i];
				
				ddElem = document.createElement("dd");
				ddElem.setAttribute("id", "object" + i);
				ddElem.az = obj.az;
				ddElem.alt = obj.alt;
				ddElem.style.top = obj.y+"px";
				ddElem.style.left = obj.x+"px";
				ddElem.style.margin = "0";
				ddElem.style.padding = "0";
				ddElem.style.position = "absolute";
				aElem = document.createElement("a");
				aElem.setAttribute("href", "javascript:void(0);");
				aElem.style.position = "absolute";
				aElem.style.textDecoration = "none";
				aElem.style.width = "5px";
				aElem.style.height = "5px";
				this.addEvtListener(aElem, "click", this.centerObjectOnClick.bind(this), false);
				this.addEvtListener(aElem, "mouseover", this.showTooltip.bind(this), false);
				this.addEvtListener(aElem, "mouseout", this.hideTooltip.bind(this), false);				
				aElem.innerHTML = "<span style='display: none;'>"+obj.name+"<br>Type: "+obj.type+"</span>";
				ddElem.appendChild(aElem); 
				skyDivImageAndMap.appendChild(ddElem);
			}			
			
			SNWEB.trackAction({action: SNWEB.consts.kTrack_ImageRequest});
			
			var imageSrcToChange = this.getHiddenImgElement();
			if(imageSrcToChange){
				imageSrcToChange.src = imageAndMapData.imageInfo.url;
			}
			// call the image refreshed callback if the user has defined one
			if (this.imageRefreshed) {
				this.imageRefreshed(); 
			}			
		},
		addEvtListener: function(domElem, event, callback, isCaptured)
		{
			if (domElem.addEventListener)
			{
				domElem.addEventListener(event, callback, isCaptured);
			}
			else if (domElem.attachEvent)
			{
				domElem.attachEvent("on"+event, callback);
			}					
		},
		removeImageMap: function()
		{
			// for this to work imageMap has to be active and swap-in DOM objects for image map and plain image have to be present
			//	In addition, image map DOM object has to be in the appropriate DIV.
			if (this.imageMap && skyDivJustImage && skyDivImageAndMap && !skyDivJustImage.parentNode && skyDivImageAndMap.parentNode)
			{
				skyDivImageAndMap.parentNode.replaceChild(skyDivJustImage, skyDivImageAndMap);
			}		
		},
		showTooltip: function(inEvt)
		{
			var myElement = ( inEvt.currentTarget ? inEvt.currentTarget : inEvt.srcElement );

			if (myElement.childNodes[0] && myElement.childNodes[0].style)
			{
				myElement.childNodes[0].style.opacity = this.tooltip.opacity;
				if (myElement.childNodes[0].style.filter !== undefined) //IE transparency
				{
					myElement.childNodes[0].style.filter = "alpha(opacity = "+ (this.tooltip.opacity * 100) +")";
				}
				myElement.childNodes[0].style.display = "block";
				myElement.childNodes[0].style.verticalAlign = "top";
				myElement.childNodes[0].style.color = this.tooltip.color;
				myElement.childNodes[0].style.backgroundColor = this.tooltip.backgroundColor;
				myElement.childNodes[0].style.fontWeight = "bold";
				myElement.childNodes[0].style.position = "absolute";
				myElement.childNodes[0].style.border = "1px solid #BCBCBC";
				myElement.childNodes[0].style.margin = "0";
				myElement.childNodes[0].style.padding = "5px";
				myElement.childNodes[0].style.width = this.tooltip.width + "px";
				
				if ((parseInt(myElement.parentNode.style.left) + 5 + this.tooltip.width) > parseInt(myElement.parentNode.parentNode.style.width))
				{
					myElement.childNodes[0].style.left = (0 - this.tooltip.width - 10) + "px";
				}
				else
				{
					myElement.childNodes[0].style.left = "+5px";
				}

				if ((parseInt(myElement.parentNode.style.top) - Element.getHeight(myElement.childNodes[0])) < 0 )
				{
					myElement.childNodes[0].style.top = "+5px";	
				}
				else
				{
					myElement.childNodes[0].style.top =  ( 0 - Element.getHeight(myElement.childNodes[0]) ) + "px";
				}

			}
		},
		hideTooltip: function(inEvt)
		{
			var myElement = ( inEvt.currentTarget ? inEvt.currentTarget : inEvt.srcElement );
			
			if (myElement.childNodes[0] && myElement.childNodes[0].style)
			{			
				myElement.childNodes[0].style.display = "none";
			}
		},
		centerObjectOnClick: function(inEvt)
		{
			var newAlt;
			var newAz;
			var myElement = ( inEvt.currentTarget ? inEvt.currentTarget : inEvt.srcElement );
			
			newAlt = myElement.parentNode.alt;
			newAz = myElement.parentNode.az;
			
			if ( (newAlt != this.situation.gaze.alt) || (newAz != this.situation.gaze.az) )
			{
				this.situation.gaze.alt = newAlt;
				this.situation.gaze.az = newAz;
				this.refreshImage();
			}
		},
		//-------------------------------------------------
		//	imageRequestFailure
		// -------------------------------------------------
		imageRequestFailure: function()
		{
			SNWEB.trackAction({action: SNWEB.consts.kTrack_ImageRequest, isError: true});
			gIgnoreClicks = false;
		},		
		//-------------------------------------------------
		//	getHiddenImgElement
		// -------------------------------------------------
		getHiddenImgElement: function()
		{
			//if($("skyDiv1") && $("skyDiv2")){
			if ($("skyDiv1").style.display == "none") {
				return $("skyImage1"); //
			}
	
			else {
				return $("skyImage2");
			}
			//}
		},	
		//-------------------------------------------------
		//	getVisibleImgElement
		// -------------------------------------------------
		getVisibleImgElement: function()
		{
			if ($("skyDiv1").style.display == "none") {
				return $("skyImage2"); //
			}
			else {
				return $("skyImage1");
			}
		},
		// -------------------------------------------------
		//	fadeUpUIElement
		// -------------------------------------------------
		fadeUpUIElement: function(intheElem)
		{
			// scriptaculous needs this to only be DOM theElem nodes!
			if (intheElem && (intheElem.nodeType === 1)) {
				Effect.Appear(intheElem, {duration: kUIElemFadeTime, to: this.controlsInFocusOpacity });
			}
		},		
		// -------------------------------------------------
		//	fadeDownUIElement
		// -------------------------------------------------
		fadeDownUIElement: function(intheElem)
		{
			// scriptaculous needs this to only be DOM theElem nodes!
			if (intheElem && (intheElem.nodeType === 1))
			{
				Effect.Fade(intheElem, {duration: kUIElemFadeTime, to: this.controlsOutOfFocusOpacity });
			}
		},
		// -------------------------------------------------
		//	switchImages
		// -------------------------------------------------
		switchImageDivs: function()
		{
			if ($("skyDiv1").style.display == "none") {
				this.showSkyDiv1();
			}
			else {
				this.showSkyDiv2();
			}
		},		
		// -------------------------------------------------
		//	showSkyDiv1
		// -------------------------------------------------
		showSkyDiv1: function(inPrefsChanged)
		{
			try {
				// Ensure the image and the DIV have the right size and position
				var skyImg = $("skyImage1");
				skyImg.setStyle(
					{
						width	: '100%',
						height	: '100%',
						top		: '0px',
						left	: '0px'
					}
				);			
			}
			catch (e) {;}
		
			var skyDiv = $("skyDiv1");
			skyDiv.setStyle(
				{
					top		: '0px',
					left	: '0px'
				}
			);
				
			//add image map if required
			if (this.imageMap && skyDivImageAndMap)
			{
				skyDivJustImage = this.getHiddenImgElement();
				skyDivJustImage.parentNode.replaceChild(skyDivImageAndMap, skyDivJustImage);				
			}
			
			Effect.Appear("skyDiv1", {afterFinish: this.listenToClicks });
			Effect.Fade("skyDiv2");	
		},		
		// -------------------------------------------------
		//	showSkyDiv2
		// -------------------------------------------------
		showSkyDiv2: function()
		{
		
			try {
				// Ensure the image and the DIV have the right size and position
				var skyImg = $("skyImage2");
				skyImg.setStyle(
					{
						width	: '100%',
						height	: '100%',
						top		: '0px',
						left	: '0px'
					}
				);			
			}
			catch (e) {;}
		
			var skyDiv = $("skyDiv2");
			skyDiv.setStyle(
				{
					top		: '0px',
					left	: '0px'
				}
			);
			
			//add image map if required
			if (this.imageMap && skyDivImageAndMap)
			{
				skyDivJustImage = this.getHiddenImgElement();
				skyDivJustImage.parentNode.replaceChild(skyDivImageAndMap, skyDivJustImage);
			}

			Effect.Appear("skyDiv2", {afterFinish: this.listenToClicks });			
			Effect.Fade("skyDiv1");
		},		
		// -------------------------------------------------
		//	listenToClicks
		// -------------------------------------------------
		listenToClicks: function()
		{
			gIgnoreClicks = false;
		},
		// -------------------------------------------------
		//	injectWidgetHTML
		// -------------------------------------------------
		injectWidgetHTML: function(inSkyViewElement)
		{
			if (inSkyViewElement)
			{
				var contentHeight	= inSkyViewElement.clientHeight;
				var contentStyle	= "height: 100%; width: 100%; position: relative; background-color: black;";
	
				var skyDivStyle		= "position: absolute; background-color: black;	overflow: hidden; top: 0px;	left: 0px; width: 100%; height: 100%;";
				var skyImageSkyle	= "position: absolute; background-color: black; overflow: hidden; top: 0px; left: 0px; width: 100%; height: 100%;";		
				
				var buttonStyle		= "position:absolute; cursor: pointer;	width: 12px; height: 12px;";
		
	
				var gazeCtrlsStyle	= "position: relative; width: 40px; height: 40px; margin-left: 2px; margin-top: 10px; margin-bottom: 2px; font-size: 0";
				var gazeUpPos		= "top: 0px; left: 14px; ";
				var gazeDownPos		= "top: 28px; left: 14px; ";
				var gazeRightPos	= "top: 14px; left: 28px; ";
				var gazeLeftPos		= "top: 14px; left: 0px; ";
				
				var northPos		= "top: 0px; left: 0px; font-size: 0;";
				var southPos		= "top: 0px; left: 14px; font-size: 0;";
				var eastPos			= "top: 0px; left: 28px; font-size: 0;";
				var westPos			= "top: 0px; left: 42px; font-size: 0;";
	
				var imageLocation 	= "http://a52.g.akamaitech.net/f/52/827/1d/www.space.com/snserver/googlegadget/images/";
				var bgImgPre		= "background-image: url(";
				var bgImgPost		= ")";
	
				var gazeUpImg		=  bgImgPre + imageLocation + "gazeUpButton.png" + bgImgPost;
				var gazeDownImg		=  bgImgPre + imageLocation + "gazeDownButton.png" + bgImgPost;
				var gazeRightImg	=  bgImgPre + imageLocation + "gazeRightButton.png" + bgImgPost;
				var gazeLeftImg		=  bgImgPre + imageLocation + "gazeLeftButton.png" + bgImgPost;
	
				var northImg		=  bgImgPre + imageLocation + "showNorthButton.png" + bgImgPost;
				var southImg		=  bgImgPre + imageLocation + "showSouthButton.png" + bgImgPost;
				var eastImg			=  bgImgPre + imageLocation + "showEastButton.png" + bgImgPost;
				var westImg			=  bgImgPre + imageLocation + "showWestButton.png" + bgImgPost;
	
				var fovDisplayStyle = "position: relative; font-family: verdana, arial, sans-serif; font-size: 10px;	font-weight: bold; text-align: center; color: white; margin-top: 6px; width: 100%;";
				var zoomButtonStyle = "position: relative; cursor: pointer; height: 12px; width: 12px; font-size: 0; margin: 4px 14px 4px 14px;"; // background: center no-repeat;
	
				var zoomInButtImg	= bgImgPre + imageLocation + "zoomInButton.png"  + bgImgPost;
				var zoomOutButtImg	= bgImgPre + imageLocation + "zoomOutButton.png"  + bgImgPost;
				
				var bgPre				= "background: url(";
				var bgPost				= ")";
				var sliderTrackTopImg	= bgPre + imageLocation + "slider-track-01.png"  + bgPost + " no-repeat";
				var sliderTrackImg		= bgPre + imageLocation + "slider-track-02.png"  + bgPost + " repeat-y";
				var sliderTrackBottImg	= bgPre + imageLocation + "slider-track-03.png"  + bgPost + " no-repeat";
				var sliderHandleImg		= bgPre + imageLocation + "slider-handle.png"  + bgPost + " no-repeat";
	
				// do this differently for IE6!
				if (navigator.appVersion.indexOf("MSIE") != -1)
				{	
					var temp=navigator.appVersion.split("MSIE");
					var version=parseFloat(temp[1]);
					if (version < 7.0)
					{
						var ieAlphaPre		= "background: none; filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='";
						var ieAlphaPost		= "', sizingMethod='crop')";
						
						gazeUpImg		= ieAlphaPre + imageLocation + "gazeUpButton.png" + ieAlphaPost;
						gazeDownImg		= ieAlphaPre + imageLocation + "gazeDownButton.png" + ieAlphaPost;
						gazeRightImg	= ieAlphaPre + imageLocation + "gazeRightButton.png" + ieAlphaPost;
						gazeLeftImg		= ieAlphaPre + imageLocation + "gazeLeftButton.png" + ieAlphaPost;
			
						northImg		=  ieAlphaPre + imageLocation + "showNorthButton.png" + ieAlphaPost;
						southImg		=  ieAlphaPre + imageLocation + "showSouthButton.png" + ieAlphaPost;
						eastImg			=  ieAlphaPre + imageLocation + "showEastButton.png" + ieAlphaPost;
						westImg			=  ieAlphaPre + imageLocation + "showWestButton.png" + ieAlphaPost;
			
						zoomInButtImg	= ieAlphaPre + imageLocation + "zoomInButton.png"  + bgImgPost;
						zoomOutButtImg	= ieAlphaPre + imageLocation + "zoomOutButton.png"  + bgImgPost;
						
						sliderTrackTopImg	= ieAlphaPre + imageLocation + "slider-track-01.png"  + bgImgPost;
						sliderTrackImg		= ieAlphaPre + imageLocation + "slider-track-02.png"  + "', sizingMethod='scale'); left: -1px;"; // different since we want to 'repeat' this one]
						sliderTrackBottImg	= ieAlphaPre + imageLocation + "slider-track-03.png"  + bgImgPost;
			
						sliderHandleImg		= ieAlphaPre + imageLocation + "slider-handle.png"  + bgImgPost;					
					}
				}
				
				// if we have a watermark
				var watermarkHTML = "";
				if (this.watermark) {
					watermarkHTML = this.watermark.getHTML();
				}
	
				// a little hack here if we're not drawing any of the controls, don't add any of the HTML
				var controlsHTML = "";
				if (this.panning || this.zooming || this.cardinals) {
					controlsHTML =		'<table style="height: 14px; width: 100%; position: absolute; bottom: 0px;" border="0" cellspacing="0" cellpadding="0">' +
										'<tr style="height: 14px">' +
											'<td align="center">' +
												'<div id="cardinalButtons" style="position: relative; width: 56px; height: 14px; font-size: 0;">' +
													'<div id="showNorthButton" class="cardinalButton" style="' + buttonStyle + northPos + northImg + '"></div>' +
													'<div id="showSouthButton" class="cardinalButton" style="' + buttonStyle + southPos + southImg + '"></div>' +
													'<div id="showEastButton" class="cardinalButton" style="' + buttonStyle + eastPos + eastImg + '"></div>' +
													'<div id="showWestButton" class="cardinalButton" style="' + buttonStyle + westPos + westImg + '"></div>' +
												'</div>' +
											'</td>' +
										'</tr>' +
									'</table>'+
									watermarkHTML +								
									'<table id="controls" border="0" cellspacing="0" cellpadding="0" style="position: absolute; height: ' + (parseInt(inSkyViewElement.offsetHeight, 10)) + 'px; width: 40px; top: 0px; left: 8px; z-index: 3;" >' +
										'<tr>' +
											'<td style="height: 40px">' +
												'<div id="gazeControls" style="' + gazeCtrlsStyle + '">' +
													'<div id="gazeUpButton" class="gazeButton" style="' + buttonStyle + gazeUpPos + gazeUpImg + '"></div>' +
													'<div id="gazeDownButton" class="gazeButton" style="' + buttonStyle + gazeDownPos + gazeDownImg + '"></div>' +
													'<div id="gazeRightButton" class="gazeButton"style="' + buttonStyle + gazeRightPos  + gazeRightImg + '"></div>' +
													'<div id="gazeLeftButton" class="gazeButton" style="' + buttonStyle + gazeLeftPos + gazeLeftImg + '"></div>' +
												'</div>' +
											'</td>' +
										'</tr>' +
										'<tr style="height: 40px">' +
											'<td style="height: 40px">' +
												'<div id="fovDisplay" style="' + fovDisplayStyle +'">fov</div>' +
												'<div id="zoomInButton" class="zoomButton" style="' + zoomButtonStyle + zoomInButtImg +'"></div>' +
												'<div id="zoomSliderTrackTop" style="position: relative; height: 3px; font-size: 0;' + sliderTrackTopImg +'"></div>' +
											'</td>' +
										'</tr>' +
										'<tr style="height: ' + (parseInt(inSkyViewElement.offsetHeight, 10) - 115) + 'px;">' +
											'<td style="height: ' + (parseInt(inSkyViewElement.offsetHeight, 10) - 115) + 'px;">' +
												'<div id="zoomSliderTrack" style="position: relative; height: 100%;' + sliderTrackImg +'">' +
													'<div id="zoomSliderHandle" style="position: relative; cursor: move; height: 8px;' + sliderHandleImg +'"></div>' +
												'</div>' +
											'</td>' +
										'</tr>' +
										'<tr style="height: 3px">' +
											'<td style="height: 3px">' +
												'<div id="zoomSliderTrackBottom" style="position: relative; height: 3px; font-size: 0;' + sliderTrackBottImg +'"></div>' +
												'<div id="zoomOutButton" class="zoomButton" style="' + zoomButtonStyle + zoomOutButtImg +'"></div>' +
											'</td>' +
										'</tr>' +
									'</table>';
				}
				
				var advertHTML = "";
				if (this.advert) {
					advertHTML = this.advert.getHTML();
				}

				var htmlString=	'<div style="' + contentStyle +'">'+
									'<div id="skyDiv" style="' + skyDivStyle + '">'+			
										'<div id="skyDiv1" style="position: absolute; top: 0px; left: 0px; width: 100%; height: 100%">' +
											'<img id="skyImage1" class="skyImage" alt="Sky View" src="' + kPlaceHolderImageURL + '" style="' + skyImageSkyle + '"/>' +
										'</div>' +
										'<div id="skyDiv2" style="display: none; position: absolute; top: 0px; left: 0px; width: 100%; height: 100%">'+
											'<img id="skyImage2" class="skyImage" alt="Sky View" src="' + kPlaceHolderImageURL + '"style="' + skyImageSkyle + '"/>' +
										'</div>'+
									'</div>'+
									controlsHTML +
									advertHTML +
								'</div>';
				
				inSkyViewElement.innerHTML = htmlString;
			}
		}
	});
		
	return new SNWebViewClass();
})();
