//------------------------------------------------------------------------------------------- 
//
// Copyright  1990-2008 by Imaginova Corp, Inc., All rights reserved.
//
// Contains:	Starry Night Widget:
//				- sends a request to the SN Server to generate a sky chart for the selected location and current time
//

/* 
Author: Antonio De Luca
// Date			Initials	Version		Comments
// ----------	---------	----------	---------------------------
// 2008/06/03	ADe			1.3.1		refactored widget with snwebview lib, key Event handlers now in snwebview lib
*/

// Author:		Ivan Pinzon
//
// Date			Initials	Version		Comments
// ----------	---------	----------	---------------------------
// 2007/09/11	IPA			1.2.1		new tracking, prefs "bug", widget title bug
// 2007/06/19	IPA			1.2			multi-widget file organization. Zoom.
// 2007/04/09	IPA			1.1			Added Quartz compositions (core image transitions), gaze control, rss ticker, new shortcuts and tabbed prefs
// 2006/04/04	IPA			1.0			New
//
//------------------------------------------------------------------------------------------- 

/*
	Status:
	FRONT:
		
		- shortcuts for:
			- ecliptic
			- constelations
			- stars
			- planets
			- az: left/right arrows ==> -/+ 30
			- alt: up/down arrows ==> +/- 10
		- darker top
	
	BACK:
		- resize window depending on div height
	
	- google analytics
	- change css font size for location depending on length
*/

// ----------------------------------------------------------------------------------------
// 										Constants:
// ----------------------------------------------------------------------------------------
SNWEB.version = "1.31";
SNWEB.platform = SNWEB.consts.kDashboardWidget;
 
var kWidgetTitle = "Starry Night Widget";

// prefs keys
var kWidgetPrefix			= "SNW_Starrynight_";
var kLocationKey 			= kWidgetPrefix + "locationKey";
var kPostalCodeKey 			= kWidgetPrefix + "postalCodeKey";
var kTimeZoneKey			= kWidgetPrefix + "timeZoneKey";
var kLatitudeKey 			= kWidgetPrefix + "latitudeKey";
var kLongitudeKey 			= kWidgetPrefix + "longitudeKey";
var kAzimuthKey 			= kWidgetPrefix + "azimuthKey";
var kShowStarLabelsKey 		= kWidgetPrefix + "showStarLabelsKey";
var kShowPlanetLabelsKey 	= kWidgetPrefix + "showPlanetLabelsKey";
var kShowConstellationsKey	= kWidgetPrefix + "showConstellationsKey";
var kShowHorizonKey 		= kWidgetPrefix + "showHorizonKey";
var kShowEclipticKey 		= kWidgetPrefix + "showEclipticKey";
var kShowDaylightKey 		= kWidgetPrefix + "showDaylightKey";
var kUseDSTKey 				= kWidgetPrefix + "useDSTKey";
var kUseLocalKey 			= kWidgetPrefix + "useLocalKey";
var kCollapsedKey 			= kWidgetPrefix + "collapsedKey";
// ----------------------------------------------------------------------------------------
// 										Global variables:
// ----------------------------------------------------------------------------------------
var gPostalCodePref = "M5C2H4";
//var gLocationPref = "Toronto, Canada";




function widgetProcessEvent(inKey)
{
	var sendRequest = true;

	switch (inKey)
	{
		case "D":
		case "d":
			setDaylight(!SNWEBVIEW.situation.options.daylight);
		break;
		
		default:
			sendRequest = false;
		break;
	}

	return sendRequest;
}


// setup is called when:
// - widget is installed (first run)
// - widget is refreshed (cmd-r)
function setup()
{
	var front= $("front");	
	
	// Google Analytics
/*
	_uacct = "UA-1172927-6";
	urchinTracker();
*/
	// hbx
	
	SNWEB.trackAction({action: SNWEB.consts.kTrack_Initialized});
		
	// create the buttons
	gInfoButton = new AppleInfoButton(document.getElementById("infoButton"), front, "black", "white", showPrefs);

	gMiniChartButton = new AppleButton(
        document.getElementById("chartDrawerButton"),
        "", // label
        20, // height
        "Images/miniChartButton.png", // left img
        "Images/miniChartButtonOn.png", // left clicked img
        20, // left width
        "", // middle img
        "", // middle clicked 
        "", // right img
        "", // right clicked img
        0, // right width
        handleChartDrawer); // callback
	
	glassFindPostalCodeButton = new AppleGlassButton(document.getElementById("findButton"), "Find", findNearestCityGoogle);
	glassDoneButton = new AppleGlassButton(document.getElementById("doneButton"), "Done", hidePrefs);

	populateAzimuthOptions();
	populateMonthsAndTime();
	
	Event.observe(window.document,"keypress", SNWEBVIEW.onKeyEvent.bind(SNWEBVIEW));	
	SNWEBVIEW.init({platform: SNWEB.consts.kDashboardWidget,
						imageMap: true,		
						version: "1.31",
						panning: true,
						zooming: true,
						cardinals: true,
						tooltip: {width:120},
						controlsOutOfFocusOpacity: 0.6,
						controlsInFocusOpacity: 1.0,
						situation: SNWEBVIEW.situation,
						advert: {beforeAdShow: cBeforeAdShow, afterAdHide: cAfterAdHide, adCfgFile: "http://www.space.com/snserver/ads/dashWidgetAds.json"}
						});
		
	$('adClick').onclick = captureFrameClick;
	
	try{
		gGeocoder = new GClientGeocoder();
		
	}
	catch(e){}
	loadPrefs();
	
	// open the drawer
	//  TODO:  change hack method of toggling? 
	var dummyEvent = document.createEvent("MouseEvents");
	gIsCollapsed = true;
	toggleView();
	sendMiniChartRequest();
	gSetupWasCalled = true;

	// start a refresh timer
    if (gRefreshTimer == null)
	{
        gRefreshTimer = setInterval("sendMiniChartRequest();", 1000 * gRefreshPeriod);
    }

	// refresh front daylight icon
	document.getElementById("daylightIconDiv").style.display = SNWEBVIEW.situation.options.daylight ? "none" : "block";
0	
	// preload the tab images
	for (i = 0; i <= kTabsCount; i++)
	{
		loadTabImages('Images/tab'+i+'.png', 'Images/tab'+i+'ON.png', i);
	}
	setTabImage("tabImg"+gCurrentTabPref, gCurrentTabPref, true);
	
	
	

}

// ----------------------------------------------------------------------------------------
//
// 											Action overrides:
//
// ----------------------------------------------------------------------------------------

function postLatLongTyping()
{
	// set the situation to these
	SNWEBVIEW.situation.location.lat = parseFloat(document.getElementById("latitudeField").value);
	SNWEBVIEW.situation.location.lng = parseFloat(document.getElementById("longitudeField").value);
}

function preFindNearestCity()
{
	// clear the postal code. User wants lat/long
	gPostalCodePref = "";
	gPostalValidated = true;
	gPostalFound = true;
}

function postFindNearestCity()
{
	gCurrentLocation = gUserLocationPref;
	setDST();
	refreshLatLongBack();
}

// ----------------------------------------------------------------------------------------
//	Find postal code
// ----------------------------------------------------------------------------------------

function postalTyping (event)
{
	gPostalValidated = false;
}

function postalKeyPress (event)
{
	switch (event.keyCode)
	{
		case 13: // return
		case 3:  // enter
		case 9:  // tab
			if (!gPostalValidated)
			{
				findPostalCode();
			}
			break;
	}
}

function findPostalCode()
{
	if (window.widget)
	{
		// get the postal code
		var postalCodeElement = document.getElementById("postalCodeField");

		if (postalCodeElement && postalCodeElement.value.length > 0)
		{
			// clean the postal code
			var postalCode = postalCodeElement.value;
			postalCode = postalCode.toUpperCase();
			postalCode = postalCode.replace(/ /g,"");
			postalCodeElement.value = postalCode;

			var url = SNWEB.getRequestURL({	type:	SNWEB.consts.kSearchPostalCodeRequest, 
												postal:		postalCode});

			// create an HTTP request to SNServer: get the lat/long for the given postal code
			gFindPostalCodeHttpRequest = new XMLHttpRequest();
			if (gFindPostalCodeHttpRequest.overrideMimeType)
			{
				gFindPostalCodeHttpRequest.overrideMimeType('text/xml');
			}
			
			// set the BACK location caption to "Searching..."
			document.getElementById("postalStatusBack").innerText = "Searching...";
	
			// send the request: ASYNC
			gFindPostalCodeHttpRequest.onreadystatechange = postalCodeCallback;
			gFindPostalCodeHttpRequest.open('GET', url, true);
			gFindPostalCodeHttpRequest.setRequestHeader("Cache-Control", "no-cache");
			gFindPostalCodeHttpRequest.send(null);
			gPostalFound = false;
		}
	}
}

function postalCodeCallback()
{
	if (gFindPostalCodeHttpRequest.readyState == 4)
	{
		if (gFindPostalCodeHttpRequest.status == 200)
		{
			var replyFields = new Array();
			replyFields = gFindPostalCodeHttpRequest.responseText.split('|');
			if (replyFields[2] != "Not Found")
			{
				gPostalFound = true;
				document.getElementById("latitudeField").value = replyFields[0];
				document.getElementById("longitudeField").value = replyFields[1];
				document.getElementById("locationBack").innerText = replyFields[2];
				SNWEBVIEW.situation.location.timezone = replyFields[3];
				document.getElementById("postalStatusBack").innerText = "";
				gLatLongValidated = true;
				savePrefs();
				setDST();
			}
			else
			{
				document.getElementById("postalStatusBack").innerText = replyFields[2];
			}

			gPostalValidated = true;
			
			delete replyFields;
		}
		else if (gFindPostalCodeHttpRequest.status == 404)
		{
			alert("URL doesn't exist");
		}
		else
		{
			alert("There was a problem with the Postal Code request: " + gFindPostalCodeHttpRequest.status);
			alert(gFindPostalCodeHttpRequest.statusText);
		}
	}
}


// ----------------------------------------------------------------------------------------
//
//												Preferences:
//
// ----------------------------------------------------------------------------------------



//----------------------------------------------------------------------
// hidePrefs() is called by the done button on the back side of the widget.  It performs the opposite transition
// as showPrefs() does.
//----------------------------------------------------------------------

function hidePrefs()
{


	if (!gLatLongValidated)
	{
		findNearestCity();
	}

	savePrefs();
//	sendMiniChartRequest();  ????????????
	
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	
	if (window.widget)
	{
		if (!gIsCollapsed)
		{
			// have to resize first
			window.resizeTo (kMaxWidgetWidth, kMaxWidgetHeight);
		}
		Event.observe(window.document,"keypress", SNWEBVIEW.onKeyEvent.bind(SNWEBVIEW));	
		widget.prepareForTransition("ToFront");		// freezes the widget and prepares it for the flip back to the front
	}
	
	back.style.display="none";			// hide the back
	front.style.display="block";		// show the front

	
	if (window.widget)
	{
		setTimeout ('widget.performTransition();', 0);		// and flip the widget back to the front
		if (gIsCollapsed)
		{
			// resize window to min size (collapsed)
			window.resizeTo (kMinWidgetWidth, kMinWidgetHeight);
		}
	}
	
	setTimeout(sendMiniChartRequest, 100);

	// restart refresh timer
	resetRefreshTimer();
	
	// restart the ticker timer
	startTickerTimer();
	restoreTickerScroll();

	// refresh front location
	document.getElementById("locationFront").innerText = gUserLocationPref;
	
	// refresh front daylight icon
	document.getElementById("daylightIconDiv").style.display = SNWEBVIEW.situation.options.daylight ? "none" : "block";
	
	refreshSituationStatus();
	setInfoButtonStyle();
	
}


//----------------------------------------------------------------------
// Load internal prefs globals with the stored pref values
//----------------------------------------------------------------------

function loadPrefs()
{
	if (window.widget)
	{
		gIsCollapsed = setPreference(widget.preferenceForKey(kCollapsedKey), kYES) == kYES ? true : false;

		gAzimuthOptionsIndexPref = setPreference(widget.preferenceForKey(kAzimuthKey), kSouthIndex);

		gUserLocationPref = setPreference(widget.preferenceForKey(kLocationKey), "Toronto, Canada");
		gPostalCodePref = setPreference(widget.preferenceForKey(kPostalCodeKey), "M5C2H4");
		
		SNWEBVIEW.situation.location.lat = setPreference(widget.preferenceForKey(kLatitudeKey), SNWEB.consts.kDefaultLatitude);
		SNWEBVIEW.situation.location.lng = setPreference(widget.preferenceForKey(kLongitudeKey), SNWEB.consts.kDefaultLongitude);
		SNWEBVIEW.situation.location.timezone = setPreference(widget.preferenceForKey(kTimeZoneKey), SNWEB.consts.kDefaultTimezone);
		SNWEBVIEW.situation.location.useDST = setPreference(widget.preferenceForKey(kUseDSTKey), kNO) == kYES ? true : false;
		SNWEBVIEW.situation.location.useTimezoneAndUTC = setPreference(widget.preferenceForKey(kUseLocalKey), kNO) == kYES ? false : true;
		SNWEBVIEW.situation.options.starLabels = setPreference(widget.preferenceForKey(kShowStarLabelsKey), kYES) == kYES ? true : false;
		SNWEBVIEW.situation.options.planetLabels = setPreference(widget.preferenceForKey(kShowPlanetLabelsKey), kYES) == kYES ? true : false;
		SNWEBVIEW.situation.options.constellations = setPreference(widget.preferenceForKey(kShowConstellationsKey), kYES) == kYES ? true : false;
		SNWEBVIEW.situation.options.horizon = setPreference(widget.preferenceForKey(kShowHorizonKey), kYES) == kYES ? true : false;
		SNWEBVIEW.situation.options.ecliptic = setPreference(widget.preferenceForKey(kShowEclipticKey), kNO) == kYES ? true : false;
		SNWEBVIEW.situation.options.daylight = setPreference(widget.preferenceForKey(kShowDaylightKey), kYES) == kYES ? true : false;
		

		
		// refresh the UI wth the prefs values
		loadPrefsUI();
		
		// set the azimuth
		setAzimuth(gAzimuthOptionsIndexPref);
		SNWEBVIEW.situation.gaze.alt = kDefaultAlt;

		gPostalFound = true;
		gLatLongValidated = true;
		gPostalValidated = true;
	}
}


//----------------------------------------------------------------------
// Set the prefs UI
//----------------------------------------------------------------------

function loadPrefsUI()
{
	document.getElementById("locationFront").innerText = gUserLocationPref;
	document.getElementById("locationBack").innerText = gUserLocationPref;
	refreshLatLongBack();
	document.getElementById("showStarLabelsChk").checked = SNWEBVIEW.situation.options.starLabels;
	document.getElementById("showLabelsChk").checked = SNWEBVIEW.situation.options.planetLabels;
	document.getElementById("showConstellationsChk").checked = SNWEBVIEW.situation.options.constellations;
	document.getElementById("showHorizonChk").checked = SNWEBVIEW.situation.options.horizon;
	document.getElementById("showEclipticChk").checked = SNWEBVIEW.situation.options.ecliptic;
	document.getElementById("showDaylightChk").checked = SNWEBVIEW.situation.options.daylight;

	document.getElementById('azimuthOptions').options[gAzimuthOptionsIndexPref].selected = true;
	document.getElementById('monthOptions').options[SNWEBVIEW.situation.dateTime.getUTCMonth()].selected = true;
	populateDays(SNWEBVIEW.situation.dateTime.getUTCFullYear(), SNWEBVIEW.situation.dateTime.getUTCMonth());
	
	document.getElementById('dayOptions').options[SNWEBVIEW.situation.dateTime.getDate()-1].selected = true;
	document.getElementById("yearField").value = SNWEBVIEW.situation.dateTime.getFullYear();

	
	var hour = SNWEBVIEW.situation.dateTime.getHours();
	var amPm = hour > 11 ? kPm : kAm;
	if (hour > 11){
		hour-= 12; 	  
	}
	hour = (hour === 0 || hour === 12) ? 11 : (hour -1);
	var localChk = document.getElementById("useLocalChk");
		
			
	document.getElementById("timeOptions").options[hour].selected = true;
	document.getElementById('amPmOptions').options[amPm].selected = true;
	document.getElementById("useDSTChk").checked = SNWEBVIEW.situation.location.useDST;
	localChk.checked =gCurrentTime;
	setLocalChk(localChk);

		
}




//----------------------------------------------------------------------
// Save the prefs
//----------------------------------------------------------------------

function savePrefs()
{
	if (window.widget)
	{
		// move the prefs values from the UI elements to our prefs vars
		savePrefsFromUI();
		
		widget.setPreferenceForKey(gUserLocationPref, kLocationKey);
		widget.setPreferenceForKey(gPostalCodePref, kPostalCodeKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.location.lat, kLatitudeKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.location.lng, kLongitudeKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.location.timezone, kTimeZoneKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.location.useDST, kUseDSTKey);
		widget.setPreferenceForKey(gCurrentTime, kUseLocalKey);
		widget.setPreferenceForKey(gAzimuthOptionsIndexPref, kAzimuthKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.starLabels, kShowStarLabelsKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.planetLabels, kShowPlanetLabelsKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.constellations, kShowConstellationsKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.horizon, kShowHorizonKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.ecliptic, kShowEclipticKey);
		widget.setPreferenceForKey(SNWEBVIEW.situation.options.daylight, kShowDaylightKey);


	}
}

//----------------------------------------------------------------------
// Get the prefs values from the UI
//----------------------------------------------------------------------

function savePrefsFromUI()
{
	// Check which side is visible and then get the values from those elements.
	// Elements from an invisible <div> return empty values
	var front = document.getElementById("front");
	var back = document.getElementById("back");
	if (back && back.style.display=="block")
	{
		// make sure I only get from the current selected tab view, otherwise I can get null or empty elements
		switch (gCurrentTabPref)
		{
			case kLocationTabView:
				gUserLocationPref = document.getElementById("locationBack").innerText;

				SNWEBVIEW.situation.location.useDST = document.getElementById("useDSTChk").checked;
				SNWEBVIEW.situation.location.useTimezoneAndUTC = document.getElementById("useLocalChk").checked;
				break;
			case kOptionsTabView:
				SNWEBVIEW.situation.options.starLabels = document.getElementById("showStarLabelsChk").checked;
				SNWEBVIEW.situation.options.planetLabels = document.getElementById("showLabelsChk").checked;
				SNWEBVIEW.situation.options.constellations = document.getElementById("showConstellationsChk").checked;
				SNWEBVIEW.situation.options.horizon = document.getElementById("showHorizonChk").checked;
				SNWEBVIEW.situation.options.ecliptic = document.getElementById("showEclipticChk").checked;
				SNWEBVIEW.situation.options.daylight = document.getElementById("showDaylightChk").checked;
				break;
			default: break;
		}
	
	}
	else if (front && front.style.display=="block")
	{
		// Will get here if the user changed the lat/long and turned to the front <div>
		// The nearest city callback needs to update the prefs.
		gUserLocationPref = document.getElementById("locationFront").innerText;
	}

}

//----------------------------------------------------------------------
// Prefs utilities
//----------------------------------------------------------------------


function setDaylight(inDaylight)
{
	SNWEBVIEW.situation.options.daylight = inDaylight;

	// refresh front daylight icon
	document.getElementById("daylightIconDiv").style.display = SNWEBVIEW.situation.options.daylight ? "none" : "block";
	
	// set info button style
	setInfoButtonStyle();
}

function setDST()
{
	SNWEBVIEW.situation.location.useDST = SNWEBVIEW.situation.currentlyExperiencingDST();
	document.getElementById("useDSTChk").checked = SNWEBVIEW.situation.location.useDST;
}

//

function dateChanged(select)
{
	// set the date in the locations array. Then save it later in the situation when hiding prefs
	var hour = 0;
	
	switch (select.id)
	{
		case "monthOptions":
			SNWEBVIEW.situation.dateTime.setUTCFullYear(document.getElementById("yearField").value);
			SNWEBVIEW.situation.dateTime.setUTCMonth(select.selectedIndex);
			populateDays(SNWEBVIEW.situation.dateTime.getFullYear(), SNWEBVIEW.situation.dateTime.getUTCMonth());
			setDST();	
			break;
		case "dayOptions":
			SNWEBVIEW.situation.dateTime.setUTCDate(select.selectedIndex+1);
			break;
		
		case "timeOptions":
		case "amPmOptions": 
		
			amPm = document.getElementById("amPmOptions").selectedIndex;
			hour = document.getElementById("timeOptions").selectedIndex + 1;
			
			if (amPm == kAm && hour > 11)
			{
				hour -= 12 ; 
				
			}
			else if (amPm == kPm && hour < 12)
			{
				hour += 12;
			}
			SNWEBVIEW.situation.dateTime.setHours(hour);
			SNWEBVIEW.situation.dateTime.setMinutes(0);
			SNWEBVIEW.situation.dateTime.setSeconds(0);
			break;
	}
	
}

function yearTyping (event)
{
	// validate year
	var year = document.getElementById("yearField").value;

	if (!isNaN(year))
	{
		// set the date in the locations array. Then save it later in the situation when hiding prefs
		SNWEBVIEW.situation.dateTime.setUTCFullYear(year);
		populateDays(SNWEBVIEW.situation.dateTime.getUTCFullYear(), SNWEBVIEW.situation.dateTime.getUTCMonth());
	}
	else
	{
		document.getElementById("yearField").value = SNWEBVIEW.situation.dateTime.getUTCFullYear();
	}
}

function yearKeyPress (event)
{
	switch (event.keyCode)
	{
		case 13: // return
		case 3:  // enter
		case 9:  // tab
			break;
	}
}

function setInfoButtonStyle()
{
	if (gIsCollapsed)
	{
		gInfoButton.setStyle("white", "white");
	}
	else if (SNWEBVIEW.situation.options.daylight)
	{
		gInfoButton.setStyle ("black", "white");
	}
	else
	{
		gInfoButton.setStyle ("white", "black");
	}
}

function setWidgetTitle()
{
	document.getElementById(kLocationFrontElemID).innerText = gCurrentLocation;
}

// ----------------------------------------------------------------------------------------
//	refreshWidgetStuff()
//
//	Purpose:	Override. Called at the end of the chart request (sendMiniChartRequest)
//				to be able to update any UI or widget specific stuff
//
//	Date		Initials	Version		Comments
//  ----------	---------	----------	---------------------------
//	2007/07/04	IPA			1.0			New
// ----------------------------------------------------------------------------------------

function refreshWidgetStuff()
{
	// make sure the handle is in sync with the FOV value
	if (Math.abs(gZoomSlider.value - SNWEBVIEW.situation.gaze.FOV) > kEpsilon)
	{
		gZoomSlider.setValue(SNWEBVIEW.situation.gaze.FOV);	
	}
}

function refreshLatLongBack()
{
	var lat = parseFloat(SNWEBVIEW.situation.location.lat);
	var lng = parseFloat(SNWEBVIEW.situation.location.lng);
	var locationStr = "Lat: " + lat.toFixed(2) + " / Long: " + lng.toFixed(2);
	document.getElementById("latLongCaptionBack").innerText = locationStr;
}

// ----------------------------------------------------------------------------------------
// 										That's all folks
// ----------------------------------------------------------------------------------------
