/*
 * Copyright 2008, Harry Kao, harry@hairycow.name
 *
 * This file is part of Time Map.
 *
 * Time Map is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Time Map is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Time Map.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 * $Rev: 851 $
 * $Author: harry $
 * $Date: 2008-09-02 22:05:36 -0700 (Tue, 02 Sep 2008) $
 * $URL: svn+ssh://nomad/export/svn/home/clock/tags/1.0.1/Time%20Map.wdgt/TimeMap.js $
 */


var MAP_SHADOW_WIDTH = 463;
var MAP_SHADOW_HEIGHT = 286;
var MAP_WIDTH = 400;
var MAP_HEIGHT = 222;
var CITY_WIDTH = 136;

/* The number of results to request for the GeoNames server. */
var MAX_ROWS = 100;

/* The maximum number of cities that we can display. */
var MAX_CITIES = 7;


/* Keep track of the state of the city pane. */
var isCityPaneOpen = false;
var isCityPaneMoving = false;

/*
 * State variables for the search form.  This is more complicated than
 * one might expect.
 *
 *   - isSearching is true if a search is in progress and false
 *     otherwise.  It's used to prevent multiple searches from
 *     occurring simultaneously.
 *
 *   - searchResults stores the results of the search.  We need a
 *     global variable for it in case the user changes the ordering.
 *     This allows us to display the list again without making another
 *     trip to the GeoNames server.
 *
 *   - wasSortButtonPressed is true if one of the sort buttons was
 *     recently clicked.  Normally, when the menu disappears, we hide
 *     the sort buttons and clear the search state.  But when the user
 *     clicks on one of the sort buttons we don't want to do this.
 *     Instead, we should update the result ordering and display the
 *     menu again.  This flag tells us not to clear the state.
 *
 *   - popupCount is the number of times the popup menu has been
 *     displayed since isSearching became true.  It's used in
 *     conjunction with wasSortButtonPressed to determine whether to
 *     clear the search state.  After the menu disappears we use a
 *     timer to schedule the state cleanup for later.  (Otherwise we'd
 *     clean up before we get a chance for the button's onmousedown
 *     handler to fire.)  However, it's possible that the entire popup
 *     process finishes before our deferred action runs, in which case
 *     wasSortButtonPressed will already have been cleared.  So if
 *     popupCount has changed then we don't clear the state.
 */
var isSearching = false;
var searchResults = null;
var wasSortButtonPressed = false;
var popupCount = 0;

/* This is true if Dashboard is active and false otherwise. */
var isWidgetActive = false;

/* An array of dictionaries representing the cities that we're
 * displaying.  See accessPrefs() for details. */
var cityList = null;

/* The 0-based index of the city that's currently selected in the
 * prefs pane table.  This will be null if no city is selected. */
var selectedCityIndex = null;

/* The city that the mouse is currently over. */
var hoverCityIndex = null;

/* The time format.  May be 'time24' or 'time12'. */
var timeFormat = null;

/* 'true' to show the time overlay or 'false' otherwise */
var showTimeOverlay = null;

/* The add and remove buttons.  We need these references later to
 * enable and disabled them. */
var addButton;
var removeButton;

/* The images that we use to represent cities.  Cities that fall on
 * the map appear as dots and ones that are too far north or south are
 * triangles. */
var cityMarkerImage = {
    'dot': new Image(),
    'up': new Image(),
    'down': new Image()
}
cityMarkerImage['dot'].src = 'city_marker.png';
cityMarkerImage['up'].src = 'city_marker_clip_top.png';
cityMarkerImage['down'].src = 'city_marker_clip_bottom.png';

/* A map from city lat/long strings to one of the following:
 *   - ['visible'] is the popup is visible and not fading in/out
 *   - ['fadeIn', animation] if the popup is fading in
 *   - ['fadeOut', animation] if the popup is fading out */
cityTimePopups = {};

/* The last time the map image was updated, in seconds since the
 * beginning of the epoch. */
var lastImageUpdateTime = 0;

/* The layer containing the city markers starts out invisible and
 * fades in when the widget initially loads.  This keeps track of
 * whether or not it's visible. */
var cityMarkerVisibility = false;

/* To prevent the map from jumping when it's updated we kind of double
 * buffer it.  There are two image elements that display the map, one
 * on top of the other.  We update the one that's not being displayed
 * and make it fade in after the new image loads. */
var currentImageBuffer = 'front';

/* true if we're currently displaying the "(city name)" hint.  This is
 * always set when the widget first loads. */
var cityHintDisplay = true;

/* true if the add button state reflects a full city list.  This will
 * briefly differ from the actual state between the removal of a city
 * and the call to updateAddButtonState(). */
var cityListFull = false;

/* this client's version string */
var versionString = null;

/* these are used to track the state of the server messages */
var messageSequenceNumber = 0;
var messageVisibility = false;
var messageUrl = '';


/* Helper function.  This is just like strcmp() in C. */
function strcmp(a, b) {
  return (a == b) ? 0 : ((a < b) ? -1 : 1);
}


/* An interpolator that computes non-linear ballistics.  min and max
 * are the limits of the range to operate over.  current is a float
 * between 0 and 1.  A linear input results in a sinusoidal output
 * from min to max. */
function sineInterpolator(min, max, current) {
  var frac = Math.sin(current * Math.PI / 2);
  return frac * (max - min) + min;
}


/* <body> onload callback. */
function setup() {

   /* we make the widget bigger than it needs to be so that popups
    * near the right side of the map don't get clipped */
   window.resizeTo(MAP_SHADOW_WIDTH + CITY_WIDTH, MAP_SHADOW_HEIGHT);

  /* info and done buttons */
  new AppleInfoButton(
      document.getElementById('infoButton'),
      document.getElementById('front'),
      'white',
      'white',
      showPrefs);
  new AppleGlassButton(
      document.getElementById('doneButton'),
      'Done',
      hidePrefs);

  /* set up info button background image */
  setupInfoBackground();

  /* add button */
  addButton = new AppleGlassButton(
      document.getElementById('addButton'),
      '+',
      addCity);
  addButton.textElement.style.width = '8px';
  addButton.textElement.style.textAlign = 'center';
  addButton.setEnabled(false);

  /* remove button */
  removeButton = new AppleGlassButton(
      document.getElementById('removeButton'),
      '\u2013',
      removeCity);
  removeButton.textElement.style.width = '8px';
  removeButton.textElement.style.textAlign = 'center';
  removeButton.setEnabled(false);

  /* read the saved preferences, update the city display, and update
   * the add button state */
  accessPrefs('read');
  updateCityDisplay();
  updateAddButtonState();

  /* check the appropriate time format button */
  document.getElementById(timeFormat).checked = true;

  /* enable the appropriate time overlay */
  var showTimeOverlayEl = document.getElementById('showTimeOverlay');
  showTimeOverlayEl.checked = (showTimeOverlay == 'true');
  onShowTimeOverlayClick(showTimeOverlayEl);

  /* read the widget version */
  var command = '/bin/sh -c "defaults read \'`pwd`/Info\' CFBundleVersion"';
  var result = widget.system(command, function() {
    var versionEl = document.getElementById('version');
    versionString = result.outputString.slice(0, -1);
    setChildText(versionEl, 'Version ' + versionString + '.');
  });

  /* set up handlers to start and stop the time updates */
  if (window.widget) {
    widget.onshow = function() {
      isWidgetActive = true;
      startTimeUpdater();
      startImageUpdater();
    };
    widget.onhide = function() {
      isWidgetActive = false;
    };
  }
}


/* Since the info button is white, it can get lost if it appears on
 * top of clouds.  We fix this by placing a dark background image
 * behind it.  This function sets up the handlers that make the
 * background image animate the same way the info button does. */
function setupInfoBackground() {

  var frontEl = document.getElementById('front');
  var backgroundEl = document.getElementById('infoButtonBackground');
  var infoButtonVisible = false;
  var currentAnimator = null;

  /* helper function that sets the opacity of the background image */
  function animHandler(animation, current, start, finish) {
    backgroundEl.style.opacity = current + '';
  }

  /* mousemove handler turns the background image on */
  frontEl.addEventListener('mousemove', function(event) {

    /* Since we've installed a mousemove handler, this function gets
     * called all the time.  If the info button is already visible
     * then we should return immediately. */
    if (infoButtonVisible) {
      return
    }

    /* if the mouseout animation is in progress, cancel it */
    if (currentAnimator) {
      currentAnimator.stop();
    }

    /* set up and start the animation */
    currentAnimator = new AppleAnimator(
        500, 50, parseFloat(backgroundEl.style.opacity), 0.5, animHandler);
    currentAnimator.oncomplete = function() {
      currentAnimator = null;
    }
    currentAnimator.start();
    infoButtonVisible = true;
  });

  /* mouseout handler turns the background image off */
  frontEl.addEventListener('mouseout', function(event) {

    /* if the mousemove animation is in progress, cancel it */
    if (currentAnimator) {
      currentAnimator.stop();
    }

    /* set up and start the animation */
    currentAnimator = new AppleAnimator(
        500, 50, parseFloat(backgroundEl.style.opacity), 0.0, animHandler);
    currentAnimator.oncomplete = function() {
        currentAnimator = null;
    }
    currentAnimator.start();
    infoButtonVisible = false;
  });
}


/* Update the displayed time.  This runs itself roughly once a minute
 * until the isWidgetActive flag is no longer true. */
function startTimeUpdater() {
  var result = widget.system('/bin/date +%S', function() {
    if (isWidgetActive) {
      var currentSeconds = parseInt(result.outputString, 10);
      var msToNextUpdate = 1000 * (60 - currentSeconds) + 50;
      if (cityList) {
        updateCityDisplay();
      }
      window.setTimeout(startTimeUpdater, msToNextUpdate);
    }
  });
}


/* Update the map image.  This happens every 5 minutes as long as the
 * widget is active.  Note:  This also initiates polling for new
 * server messages too.  Maybe that functionality should go into its
 * own function... */
function startImageUpdater() {

  var updatePeriod = 5 * 60;  /* seconds */
  var url = 'http://hairycow.name/projects/time_map/current_small.jpg';

  var result = widget.system('/bin/date +%s', function() {
    if (isWidgetActive) {
      var secsSinceEpoch = parseInt(result.outputString, 10);
      if (secsSinceEpoch >= (lastImageUpdateTime + updatePeriod)) {

        /* map image */
        var imageId = (currentImageBuffer == 'front') ?
            'mapImageBack' : 'mapImageFront';
        var imageEl = document.getElementById(imageId);
        imageEl.onload = mapImageFader;
        imageEl.src = url + '?' + secsSinceEpoch;
        lastImageUpdateTime = secsSinceEpoch;

        /* message */
        updateMessage();
      }
      var msToNextUpdate =
          1000 * (lastImageUpdateTime + updatePeriod - secsSinceEpoch);
      window.setTimeout(startImageUpdater, msToNextUpdate);
    }
  });
}


/* Poll the server for the latest message and display it if we get
 * something back. */
function updateMessage() {
  var url = 'http://hairycow.name/projects/time_map/message.cgi?' +
      'v=' + encodeURIComponent(versionString) +
      '&seq=' + encodeURIComponent(messageSequenceNumber);
  var request = new XMLHttpRequest();
  request.onload = function() {
    if (request.status == 200) {
      try {
        eval('var message = ' + request.responseText);
        if (message['valid'] == 'true') {
          messageSequenceNumber = message['seq'];
          displayMessage(message['msg'], message['url']);
        }
      }
      catch (e) {
      }
    }
  };
  request.open('GET', url);
  request.send(null);
}


/* Display a message from the server. */
function displayMessage(message, url) {

  messageUrl = url;
  setChildText(document.getElementById('serverMessageText'), message);
  document.getElementById('serverMessage').style.cursor =
      (url) ? 'pointer' : 'default';

  if (!messageVisibility) {
    var messageEl = document.getElementById('serverMessageContainer');
    var animator = new AppleAnimator(
        500, 50, 0, 1,
        function(animation, current, start, finish) {
          messageEl.style.opacity = current + '';
        }
    );
    messageVisibility = true;
    messageEl.style.visibility = 'visible';
    animator.start();
  }
}


/* An onload handler for the map image that performs the fade. */
function mapImageFader() {

  function animHandler(animation, current, start, finish) {
    frontImage.style.opacity = current + '';
  }

  var startOpacity = (currentImageBuffer == 'front') ? 1 : 0;
  var endOpacity = (currentImageBuffer == 'front') ? 0 : 1;
  var frontImage = document.getElementById('mapImageFront');

  new AppleAnimator(1000, 50, startOpacity, endOpacity, animHandler).start();
  currentImageBuffer = (currentImageBuffer == 'front') ? 'back' : 'front';
}


/* Read in the saved city list or write it out, depending on the value
 * of the write flag.  The data is read into or written from the
 * global cityList variable. */
function accessPrefs(mode) {

  /* a list of keys that we'll read or write */
  var cityKeys = [
      'name',
      'location',
      'timezone',
      'latitude',
      'longitude',
  ];

  if (window.widget) {

    /* write */
    if (mode == 'write') {

      /* time format and overlay */
      widget.setPreferenceForKey(timeFormat, 'timeFormat');
      widget.setPreferenceForKey(showTimeOverlay, 'showTimeOverlay');

      /* city list */
      widget.setPreferenceForKey(cityList.length, 'cityCount');
      for (var i = 0; i < cityList.length; ++i) {
        for (var j = 0; j < cityKeys.length; ++j) {
          var key = cityKeys[j];
          widget.setPreferenceForKey(cityList[i][key], key + '-' + i);
        }
      }
    }

    /* read */
    else {

      /* time format */
      timeFormat = widget.preferenceForKey('timeFormat');
      if (timeFormat == undefined) {
        timeFormat = 'time24';
      }

      /* time overlay */
      showTimeOverlay = 'false';
//      showTimeOverlay = widget.preferenceForKey('showTimeOverlay');
//      if (showTimeOverlay == undefined) {
//        showTimeOverlay = 'true';
//      }

      /* determine how many cities we have */
      var count = widget.preferenceForKey('cityCount');
      cityList = []

      /* if we've never configured the city list, create a default */
      if (count == undefined) {
        cityList.push({
            'name': 'San Francisco',
            'location': 'United States > California',
            'timezone': 'America/Los_Angeles',
            'latitude': 37.775,
            'longitude': -122.419
        });
      }

      /* read in the cities */
      else {
        for (var i = 0; i < count; ++i) {
          cityList.push({});
          for (var j = 0; j < cityKeys.length; ++j) {
            var key = cityKeys[j];
            cityList[i][key] = widget.preferenceForKey(key + '-' + i);
          }
        }
      }
    }
  }
}


/*
 * Convert (lat, long) to pixel coordinates on the map.  This returns
 * a dictionary with the following entries:
 *
 *   x is the x-coordinate.
 *
 *   y is the y-coordinate.  This is clipped if the city does not fit
 *   on the map.
 *
 *   yRaw is the unclipped y-coordinate.  This is the same as y if the
 *   city fits on the map.
 */
function latLongToXY(lat, lng) {

  /* Since the projection necessarily cuts off the north and south
   * poles we need to scale the calculated y-values.  This should
   * really depend on how much of the map was chopped off but the
   * numbers were actually determined empirically. */
  if (latLongToXY.latExtrema == undefined) {
    //var latRad = Math.PI * (90 - 15)/ 180;
    var latRad = Math.PI * 88.2 / 180;
    latLongToXY.latExtrema =
        Math.log(Math.tan(latRad) + 1 / Math.cos(latRad));
  }

  /* The left edge of the map seems to represent -170 degrees instead
   * of -180 degrees as one might expect. */
  //var x = (lng + 180) * MAP_WIDTH / 360;
  var x = ((lng + 170 + 360) % 360) * MAP_WIDTH / 360;
  x = Math.max(0, x);
  x = Math.min(x, MAP_WIDTH);

  var latRad = Math.PI * lat / 180;
  var yAbs = Math.log(Math.tan(latRad) + 1 / Math.cos(latRad));
  var yRel = yAbs / latLongToXY.latExtrema;
  //var y = MAP_HEIGHT - (MAP_HEIGHT * yRel + (MAP_HEIGHT / 2));
  var yRaw = MAP_HEIGHT * ((1 / 2) - yRel);
  var y = Math.max(0, yRaw);
  y = Math.min(y, MAP_HEIGHT);

  return {'x': x, 'y': y, 'yRaw': yRaw};
}


/* Update the city lists on both the front and back sides.  This
 * populates the lists with the current values from the cityList
 * variable as well as the current time. */
function updateCityDisplay() {

  /*
   * Parse a line from zdump, which looks something like this:
   *
   *   UTC  Thu Jul  3 07:10:52 2008 UTC
   *
   * Return a dictionary containing the following keys:
   *
   *   - dayOfWeekShort: 'Mon', 'Tue', etc.
   *   - dayOfWeekLong: 'Monday', 'Tuesday', etc.
   *   - secsSinceMidnight: The number of seconds elapsed since 00:00.
   *   - utcOffsetSecs:  Delta from UTC.  May be positive or negative.
   *   - utcOffsetString:  Human-readable UTC offset.
   *   - dateString24:  24-hour time.
   *   - dateString12:  12-hour time.
   *
   * utcInfo, if supplied, should be the dictionary corresponding to
   * the UTC line.  It's used to calculate the UTC offset.
   */
  function parseZdumpLine(line, utcInfo) {

    var daysOfWeek = {
        'Mon': [0, 'Monday'],
        'Tue': [1, 'Tuesday'],
        'Wed': [2, 'Wednesday'],
        'Thu': [3, 'Thursday'],
        'Fri': [4, 'Friday'],
        'Sat': [5, 'Saturday'],
        'Sun': [6, 'Sunday']
    }

    /*
     * This regex captures information in the following array indices:
     *
     * 1: day of week
     * 2: month
     * 3: day of month
     * 4: hours
     * 5: minutes
     * 6: seconds
     */
    var regex = /\S+\s+(\S+) (\S+)\s+(\d+) (\d\d):(\d\d):(\d\d)/;
    var parts = regex.exec(line);

    var hours = parseInt(parts[4], 10);
    var mins = (hours * 60) + parseInt(parts[5], 10);
    var secs = (mins * 60) + parseInt(parts[6], 10);

    var info = {
        'dayOfWeekShort': parts[1],
        'dayOfWeekLong': daysOfWeek[parts[1]][1],
        'secsSinceMidnight': secs,
        'dateString24': parseInt(parts[4], 10) + ':' + parts[5]
    };

    /* calculate the offset from UTC, in seconds */
    if (utcInfo) {
      var utcDayOfWeek = utcInfo['dayOfWeekShort'];
      info['utcOffsetSecs'] = secs - utcInfo['secsSinceMidnight'];
      if (parts[1] != utcDayOfWeek) {
        if ((daysOfWeek[parts[1]][0] + 1) % MAX_CITIES ==
            daysOfWeek[utcDayOfWeek][0]) {
          info['utcOffsetSecs'] -= 86400;
        }
        else {
          info['utcOffsetSecs'] += 86400;
        }
      }
    }
    else {
      info['utcOffsetSecs'] = 0;
    }

    /* the UTC offset in [+|-]HH:MM format */
    info['utcOffsetString'] =
        (info['utcOffsetSecs'] / 3600) + ':' +
        ((info['utcOffsetSecs'] % 3600) / 60);

    /* human-readable time in 12-hour format */
    var hours = parseInt(parts[4], 10);
    var period = (hours < 12) ? 'AM' : 'PM';
    if (hours == 0) {
      hours = 12;
    }
    else if (hours > 12) {
      hours -= 12;
    }
    info['dateString12'] = hours + ':' + parts[5] + ' ' + period;

    return info;
  }

  /* create a list of timezones that we care about */
  var timezones = ['UTC']
  for (var i = 0; i < cityList.length; ++i) {
    timezones.push(cityList[i]['timezone']);
  }

  /* Run zdump to determine the current time in each timezone.  We do
   * this asynchronously because widget.system() tends to screw up the
   * entire Dashboard otherwise when something goes wrong. */
  var command = '/usr/sbin/zdump ' + timezones.join(' ');
  var result = widget.system(command, function() {

    /* Create an array containing one output string per timezone.  The
     * first will be UTC.  The last is empty because the command's
     * output ends with a newline. */
    var timeList = result.outputString.split('\n');
    var utcInfo = parseZdumpLine(timeList[0]);

    /* move the time overlay */
    {
      /* retrieve the current slider position */
      var timeOverlaySlider = document.getElementById('timeOverlaySlider');
      var oldLeft = timeOverlaySlider.style.left;
      if (oldLeft == '') {
        oldLeft = '0px';
      }
      oldLeft = parseInt(oldLeft.slice(0, -2), 10);

      /* compute the new position of the slider */
      var dayFrac = utcInfo['secsSinceMidnight'] / (24. * 3600);
      var newLeft = -dayFrac * MAP_WIDTH - 252;
      /* always want to move left */
      if (oldLeft < newLeft) {
        newLeft -= MAP_WIDTH;
      }
      /* move the minimum distance */
      if (newLeft < (oldLeft - MAP_WIDTH)) {
        newLeft += MAP_WIDTH;
      }

      /* animate the slider */
      new AppleAnimator(2000, 50, 0, 1, function(anim, cur, start, end) {
        var left = sineInterpolator(oldLeft, newLeft, cur);
        if (left < -MAP_WIDTH) {
          left += MAP_WIDTH;
        }
        timeOverlaySlider.style.left = left + 'px';
      }).start();
    }

    /* Attach the time information to each city record and sort by UTC
     * offset.  We do this every time we update the time because the
     * ordering may change for various reasons (a city was added, DST
     * change, etc.). */
    for (var i = 0; i < cityList.length; ++i) {
      cityList[i]['timeInfo'] = parseZdumpLine(timeList[i + 1], utcInfo);
    }
    cityList.sort(function(a, b) {
      var aUtcOffSecs = a['timeInfo']['utcOffsetSecs'];
      var bUtcOffSecs = b['timeInfo']['utcOffsetSecs'];
      if (aUtcOffSecs != bUtcOffSecs) {
        return aUtcOffSecs - bUtcOffSecs;
      }
      else {
        return strcmp(a['name'], b['name']);
      }
    });

    /* call the helper function that actually does the update */
    updateCities();
  });
}


/* Update the city lists on both sides as well as the markers on the
 * map. */
function updateCities() {

  /* for the map on the front panel */
  var canvas = document.getElementById('cityMarkerOverlay');
  var context = canvas.getContext('2d');
  context.clearRect(0, 0, MAP_WIDTH + 20, MAP_HEIGHT);

  /* these are used to update the prefs table */
  var table = document.getElementById('cityTable');
  var trList = table.getElementsByTagName('TR');

  /* update each row */
  for (var i = 0; i < MAX_CITIES; ++i) {

    var city = cityList[i];

    /* popup */
    if (i < cityList.length) {
      var popupId = 'popup_' + city['latitude'] + '_' + city['longitude'];
      var value = cityTimePopups[popupId];
      if (value) {
        var popupContainerEl = document.getElementById(popupId);
        var timeContainerEl = popupContainerEl.childNodes(1).childNodes(1);
        setChildText(timeContainerEl, makeTimeString(city));
      }
    }

    /* front panel */
    var cityDiv = document.getElementById('city-' + i);
    var timeDiv = document.getElementById('time-' + i);
    if (i < cityList.length) {

      /* city list */
      setChildText(cityDiv, city['name']);
      setChildText(timeDiv, makeTimeString(city));
      timeDiv.className = 'time';

      /* city marker */
      var coords = latLongToXY(city['latitude'], city['longitude']);
      if (coords.y == coords.yRaw) {
        var imageKey = 'dot';
      }
      else {
        var imageKey = (coords.y > coords.yRaw) ? 'up' : 'down';
      }
      context.drawImage(
          cityMarkerImage[imageKey], coords.x - 6, coords.y - 6);
    }
    else {
      setChildText(cityDiv, '\u00A0');
      setChildText(timeDiv, '\u00A0');
      timeDiv.className = 'time empty';
    }

    /* back panel */
    var tdList = trList[i].getElementsByTagName('TD');
    if (i < cityList.length) {
      setChildText(tdList[0], city['name']);
      setChildText(tdList[1], city['location']);
    }
    else {
      setChildText(tdList[0], '\u00A0');
      setChildText(tdList[1], '\u00A0');
    }
  }

  if (!cityMarkerVisibility) {
    function animHandler(animation, current, start, finish) {
      overlay.style.opacity = current + '';
    }
    var overlay = document.getElementById('cityMarkerOverlay');
    new AppleAnimator(1000, 50, 0, 1, animHandler).start();
    cityMarkerVisibility = true;
  }
}


/* Given a city object, create a string of the form:
 *
 *   "[day of week], [time]"
 *
 * The time is in 12- or 24-hour format, depending on the user's
 * preferences. */
function makeTimeString(city) {
  var day = city['timeInfo']['dayOfWeekLong'] + ', ';
  if (timeFormat == 'time24') {
    return day + city['timeInfo']['dateString24'];
  }
  else {
    return day + city['timeInfo']['dateString12'];
  }
}


/* Boilerplate method for flipping over to the back side. */
function showPrefs() {

  if (window.widget) {
    widget.prepareForTransition('ToBack');
  }

  document.getElementById('front').style.display = 'none';
  document.getElementById('back').style.display = 'block';

  if (window.widget) {
    setTimeout('widget.performTransition();', 0);
  }
}


/* Boilerplate method for flipping over to the front side. */
function hidePrefs() {

  if (window.widget) {
    widget.prepareForTransition('ToFront');
  }

  document.getElementById('front').style.display = 'block';
  document.getElementById('back').style.display = 'none';

  /* For some reason the city markers get cleared on 10.4 when we flip
   * back to the front side.  Redraw them here before the transition. */
  updateCityDisplay();

  if (window.widget) {
    setTimeout('widget.performTransition();', 0);
  }
}


/* Handler for mouse moves on the transparent DIV overlaying the map.
 * This turns the city name/time popup on and off. */
function onMapMouseMove(event) {
  for (var i = 0; i < cityList.length; ++i) {

    var city = cityList[i];
    var coords = latLongToXY(city['latitude'], city['longitude']);
    var deltaSq = Math.pow(coords.x - event.layerX, 2) +
        Math.pow(coords.y - event.layerY, 2);

    var popupId = 'popup_' + city['latitude'] + '_' + city['longitude'];
    var value = cityTimePopups[popupId];

    /* if we're within a certain threshold, show the popup */
    if (deltaSq <= 36) {

      /* if the popup doesn't exist, create it and fade in */
      if (value === undefined) {

        var cityContainer = document.createElement('div');
        cityContainer.className = 'city';
        cityContainer.appendChild(
            document.createTextNode(city['name']));

        var timeContainer = document.createElement('div');
        timeContainer.className = 'time';
        timeContainer.appendChild(
            document.createTextNode(makeTimeString(city)));

        var textContainer = document.createElement('div');
        textContainer.className = 'textContainer';
        textContainer.appendChild(cityContainer);
        textContainer.appendChild(timeContainer);

        var leftCapCity = document.createElement('div');
        leftCapCity.className = 'city';
        leftCapCity.appendChild(document.createTextNode('\u2009'));
        var leftCapTime = document.createElement('div');
        leftCapTime.className = 'time';
        leftCapTime.appendChild(document.createTextNode('\u2009'));
        var leftCap = document.createElement('div');
        leftCap.className = 'leftCap';
        leftCap.appendChild(leftCapCity);
        leftCap.appendChild(leftCapTime);

        var rightCapCity = document.createElement('div');
        rightCapCity.className = 'city';
        rightCapCity.appendChild(document.createTextNode('\u2009'));
        var rightCapTime = document.createElement('div');
        rightCapTime.className = 'time';
        rightCapTime.appendChild(document.createTextNode('\u2009'));
        var rightCap = document.createElement('div');
        rightCap.className = 'rightCap';
        rightCap.appendChild(rightCapCity);
        rightCap.appendChild(rightCapTime);

        var popupContainer = document.createElement('div');
        popupContainer.id = popupId;
        popupContainer.className = 'cityPopup';
        popupContainer.style.left = (coords.x + 8) + 'px';
        popupContainer.style.top = (coords.y - 14) + 'px';
        popupContainer.appendChild(leftCap);
        popupContainer.appendChild(textContainer);
        popupContainer.appendChild(rightCap);

        var popupOverlay = document.getElementById('popupOverlay');
        popupOverlay.appendChild(popupContainer);

        var animator = animateCityPopup(popupId, 1);
        cityTimePopups[popupId] = ['fadeIn', animator];
      }

      /* if the popup is fading out, abort animation and fade in */
      else if (value[0] == 'fadeOut') {
        value[1].stop();
        var animator = animateCityPopup(popupId, 1);
        cityTimePopups[popupId] = ['fadeIn', animator];
      }
    }

    /* if we've moved away from the marker, hide the popup */
    else if (value) {
      if (value[0] == 'visible') {
        var animator = animateCityPopup(popupId, 0);
        cityTimePopups[popupId] = ['fadeOut', animator];
      }
      else if (value[0] == 'fadeIn') {
        value[1].stop();
        var animator = animateCityPopup(popupId, 0);
        cityTimePopups[popupId] = ['fadeOut', animator];
      }
    }
  }
}


/* If the mouse leaves the DIV we might not get a mousemove event
 * indicating that we're no longer in the vicinity of a marker.  So
 * loop through the list of popups and turn them all off. */
function onMapMouseOut(event) {
  for (var i = 0; i < cityList.length; ++i) {

    var city = cityList[i];
    var coords = latLongToXY(city['latitude'], city['longitude']);
    var popupId = 'popup_' + city['latitude'] + '_' + city['longitude'];
    var value = cityTimePopups[popupId];

    if (value && (value[0] != 'fadeOut')) {
      if (value[0] != 'visible') {
        value[1].stop();
      }
      var animator = animateCityPopup(popupId, 0);
      cityTimePopups[popupId] = ['fadeOut', animator];
    }
  }
}


/* Fade the popup in or out.  endOpacity shoud be 0 to fade out and 1
 * to fade in.  We take care of the cityTimePopups state and delete
 * the popup element if necessary.  Returns the animation object. */
function animateCityPopup(id, endOpacity) {
  var popupContainer = document.getElementById(id);
  var startOpacity = (popupContainer.style.opacity) ?
      parseFloat(popupContainer.style.opacity) : 0;
  var animator = new AppleAnimator(
      500, 50, startOpacity, endOpacity,
      function(animation, current, start, finish) {
        popupContainer.style.opacity = current + '';
      }
  );
  animator.oncomplete = function() {
    if (endOpacity == 0) {
      popupContainer.parentNode.removeChild(popupContainer)
      delete cityTimePopups[id];
    }
    else {
      cityTimePopups[id] = ['visible'];
    }
  }
  animator.start();
  return animator;
}


/* Open or close the city list panel on the front side. */
function toggleCityPane() {

  var shadowRight = document.getElementById('dropShadowRight');
  var cityListContainer = document.getElementById('cityListContainer');

  function animateCityPane(start, end) {

    function animHandler(animation, current, start, finish) {
      cityListContainer.style.right = '-' + current + 'px';
      shadowRight.style.width = (63 + current) + 'px';
    }

    animator = new AppleAnimator(300, 20);
    animator.addAnimation(new AppleAnimation(start, end, animHandler));
    animator.oncomplete = function() {
      if (isCityPaneOpen) {
        cityListContainer.style.visibility = 'hidden';
        if (window.widget) {
//          window.resizeTo(MAP_SHADOW_WIDTH, MAP_SHADOW_HEIGHT);
        }
      }
      isCityPaneOpen = !isCityPaneOpen;
      isCityPaneMoving = false;
    };
    animator.start();
  }

  /* don't do anything if we're in the middle of a move */
  if (isCityPaneMoving) {
    return;
  }
  else {
    isCityPaneMoving = true;
  }

  if (isCityPaneOpen) {
    animateCityPane(CITY_WIDTH, 0);
  }
  else {
    if (window.widget) {
//      window.resizeTo(MAP_SHADOW_WIDTH + CITY_WIDTH, MAP_SHADOW_HEIGHT);
    }
    cityListContainer.style.visibility = 'visible';
    animateCityPane(0, CITY_WIDTH);
  }
}


/* Set the state of the add button.  This also makes adjustmets to the
 * city name text field. */
function updateAddButtonState() {

  var cityNameEl = document.getElementById('cityName');

  /* if we're in the middle of a search we should disable both the add
   * button and the text field */
  if (isSearching) {
    cityNameEl.disabled = true;
    addButton.setEnabled(false);
  }

  /* if the city list is full, disable the add button and text field
   * and also provide a hint to the user */
  else if (cityList.length == MAX_CITIES) {
    addButton.setEnabled(false);
    cityNameEl.disabled = true;
    cityNameEl.value = '(city list full)';
    cityNameEl.blur();
    cityListFull = true;
  }

  /* common case: not searching and not full */
  else if (cityList.length < MAX_CITIES) {

    /* if the list was full, clear it */
    if (cityListFull) {
      cityNameEl.value = '';
      cityListFull = false;
    }

    /* don't enable the add button if the user has never focused it */
    addButton.setEnabled(!cityHintDisplay && !!cityNameEl.value);
    cityNameEl.disabled = false;
  }
}


/* Clear the "(city name)" hint if it's being displayed. */
function onCityNameFocus(cityNameEl) {
  if (cityHintDisplay) {
    cityNameEl.value = '';
    cityHintDisplay = false;
  }
}


/* Handle key presses in the city field.  We update the button state,
 * clear the status message, and initiate the search if the user
 * pressed Enter. */
function onCityNameKeyUp(event, cityNameEl) {
  updateAddButtonState();
  setChildText(document.getElementById('statusMessage'), '');
  if (cityNameEl.value && event && (event.keyCode == 13)) {
    addCity();
  }
}


/* When this fires the popup menu will have recently closed.  We set
 * the wasSortButtonPressed flag to tell the popup function not to
 * clear the search state.  We also add an onmouseup handler for the
 * body to clear the search state in case the user clicks on the
 * button, drags off of it, and releases. */
function onSortMouseDown(event) {
//  alert('in button onmousedown()');
  wasSortButtonPressed = true;
  document.body.onmouseup = function() {
//    alert('in body onmouseup()');
    document.body.onmouseup = null;
    clearSearchState();
  }
}


/* We really ought to do nothing but clear the body onmouseup handler
 * and wait for the onclick here.  But on really fast clicks we
 * sometimes don't get an onmousedown action (I don't know why) so set
 * wasSortButtonPressed here too, just in case. */
function onSortMouseUp(event) {
//  alert('in button onmouseup()');
  wasSortButtonPressed = true;
  document.body.onmouseup = null;
}


/* Re-display the popup menu. */
function onSortClick(event) {
//  alert('in button onclick()');
  wasSortButtonPressed = false;
  popupMenu();
}


/* Change the time format. */
function onTimeFormatClick(radioButtonEl) {
  if (timeFormat != radioButtonEl.value) {
    radioButtonEl.checked = true;
    timeFormat = radioButtonEl.value;
    accessPrefs('write');
    accessPrefs('read');
    updateCityDisplay();
    onShowTimeOverlayClick(document.getElementById('showTimeOverlay'));
  }
}


/* Toggle the time overlay. */
function onShowTimeOverlayClick(checkboxEl) {
  if (checkboxEl.checked) {
    if (timeFormat == 'time12') {
      document.getElementById('timeOverlay12').style.display = 'block';
      document.getElementById('timeOverlay24').style.display = 'none';
    }
    else {
      document.getElementById('timeOverlay12').style.display = 'none';
      document.getElementById('timeOverlay24').style.display = 'block';
    }
    showTimeOverlay = 'true';
  }
  else {
    document.getElementById('timeOverlay12').style.display = 'none';
    document.getElementById('timeOverlay24').style.display = 'none';
    showTimeOverlay = 'false';
  }
  accessPrefs('write');
  accessPrefs('read');
}


/* Handler for the mousemove event on the city list.  This highlights
 * the city that we're hovering over and draws an arc to its location
 * on the map. */
function onCityListMouseMove(event) {
  var index = Math.floor((event.layerY - 4)/ 32);
  if ((index >= 0) && (index != hoverCityIndex)) {

    /* restore original state */
    onCityListMouseOut(event);

    if (index < cityList.length) {

      /* calculate arc start/end points */
      var listX = MAP_WIDTH + 4;
      var listY = 32 * index + 14;
      var coords = latLongToXY(
          cityList[index]['latitude'], cityList[index]['longitude']);

      /* draw arc */
      var canvas = document.getElementById('cityArcOverlay');
      var context = canvas.getContext('2d');
      context.beginPath();
      context.moveTo(listX, listY);
      context.strokeStyle = 'rgba(255, 255, 255, 0.75)';
      context.lineWidth = 1.5;
      context.quadraticCurveTo(
          (coords.x + listX) / 2, listY, coords.x, coords.yRaw);
      context.stroke();

      /* highlight the row that we're hovering over */
      var cityContainerNew = document.getElementById(
          'cityContainer-' + index);
      cityContainerNew.className = 'cityContainerHover';
    }
    hoverCityIndex = index;
  }
}


/* Clear the arcs and highlighting. This restores the widget to the
 * non-hover state. */
function onCityListMouseOut(event) {

  /* clear the existing arc */
  var canvas = document.getElementById('cityArcOverlay');
  var context = canvas.getContext('2d');
  context.clearRect(0, 0, MAP_WIDTH + 20, MAP_HEIGHT);

  /* restore city container color */
  if ((hoverCityIndex != null) && (hoverCityIndex < cityList.length)) {
    var cityContainerOld = document.getElementById(
        'cityContainer-' + hoverCityIndex);
    cityContainerOld.className = '';
  }

  hoverCityIndex = null;
}


/* Hide the message. */
function onServerMessageCloseButtonClick() {
  var messageEl = document.getElementById('serverMessageContainer');
  var animator = new AppleAnimator(
      500, 50, 1, 0,
      function(animation, current, start, finish) {
        messageEl.style.opacity = current + '';
      }
  );
  animator.oncomplete = function() {
    messageEl.style.visibility = 'hidden';
    messageVisibility = false;
  }
  animator.start();
}


/* If the message has a URL, launch it and hide the message.
 * Otherwise do nothing. */
function onServerMessageClick() {
  if (messageUrl) {
    widget.openURL(messageUrl);
    onServerMessageCloseButtonClick();
  }
}


/* onclick handler for the add button.  This initiates the city search
 * and selection code, which is surprisingly complicated... */
function addCity() {

  /* don't do anything if we're already searching */
  if (isSearching) {
    return;
  }
  else {
    isSearching = true;
    updateAddButtonState();
    setChildText(document.getElementById('statusMessage'), 'Searching...');
  }

  /* send an AJAX request */
  var cityName = document.getElementById('cityName').value;
  var url = 'http://ws.geonames.org/search?featureClass=P' +
      '&type=json&style=full&name=' + encodeURIComponent(cityName) +
      '&maxRows=' + encodeURIComponent(MAX_ROWS);
  var request = new XMLHttpRequest();
  request.onload = function() {
    if (request.status != 200) {
      clearSearchState('Error contacting name database.');
    }
    else {
      try {

        eval('var data = ' + request.responseText);
        handleCityQueryResponse(data);

        /* If the prefs pane is not being displayed, the user must
         * have flipped to the front while we were waiting for the
         * AJAX response.  In this case, abort the search. */
        if (document.getElementById('back').style.display == 'none') {
          alert('search aborted');
          clearSearchState();
        }
        else if (searchResults.length > 0) {

          /* Hide the status message and enable the search result
           * sorting controls.  We used to do this just prior to
           * displaying the menu but this doesn't work under 10.4.  My
           * theory is that 10.4 doesn't perform DOM manipulation in a
           * blocking manner.  Rather, it waits for the current event
           * to finish processing.  The popup menu itself blocks the
           * event processing though, which is what prevents the sort
           * controls from showing up.  The solution is to show the
           * controls immediately and schedule the menu to be
           * displayed later. */
          document.getElementById('statusMessage').style.display = 'none';
          document.getElementById('sortControls').style.display = 'block';
          window.setTimeout(function() {
            popupMenu();
          }, 0);
        }
        else {
          clearSearchState('No matches.');
        }
      }
      catch (e) {
        clearSearchState('Error processing search results.');
      }
    }
  };
  request.open('GET', url);
  request.send(null);
}


/* Save the query results in the global variable searchResults. */
function handleCityQueryResponse(data) {

  /* filter out records whose population == 0 */
  searchResults = []
  var resultList = data['geonames'];
  for (var i = 0; i < resultList.length; ++i) {
    if (resultList[i]['population'] != 0) {
      searchResults.push(resultList[i]);
    }
  }
}


/* Display the menu containing the search results.  This sorts the
 * returned list before creating the menu.  It also processes the
 * selection. */
function popupMenu() {

  ++popupCount;
  var popupCountStart = popupCount;

  /* determine sort order */
  var populationButton = document.getElementById('sortButtonPopulation');
  var sortOrder = populationButton.checked ? 'population' : 'country';

  /* a list of human-friendly strings corresponding to menu items */
  var displayStringList = [];

  /* sort by country */
  if (sortOrder == 'country') {

    /* compares from country > ... > city */
    function lexicalCmp(a, b) {
      if (a['countryName'] != b['countryName']) {
        return strcmp(a['countryName'], b['countryName']);
      }
      else if (a['adminName1'] != b['adminName1']) {
        return strcmp(a['adminName1'], b['adminName1']);
      }
      else {
        return strcmp(a['name'], b['name']);
      }
    }

    searchResults.sort(lexicalCmp);
    for (var i = 0; i < searchResults.length; ++i) {
      var displayString = searchResults[i]['countryName'];
      if (searchResults[i]['adminName1']) {
        displayString += ' > ' + searchResults[i]['adminName1'];
      }
      displayString += ' > ' + searchResults[i]['name'];
      displayStringList.push(displayString);
    }
  }

  /* sort by population */
  else {

    /* population comparator */
    function populationCmp(a, b) {
      return b['population'] - a['population'];
    }

    searchResults.sort(populationCmp);
    for (var i = 0; i < searchResults.length; ++i) {
      var displayString = searchResults[i]['name'];
      if (searchResults[i]['adminName1']) {
        displayString += ', ' + searchResults[i]['adminName1'];
      }
      displayString += ', ' + searchResults[i]['countryName'];
      displayStringList.push(displayString);
    }
  }

  /* create the menu */
  var menu = widget.createMenu();
  for (var i = 0; i < displayStringList.length; ++i) {
    menu.addMenuItem(displayStringList[i]);
  }

  /* show the menu and process the selection */
  var selectedItem = menu.popup(39, 188);
  if (selectedItem >= 0) {

    /* add a new entry to cityList */
    var selection = searchResults[selectedItem];
    var location = selection['countryName'];
    if (selection['adminName1']) {
      location += ' > ' + selection['adminName1'];
    }
    cityList.push({
        'name': selection['name'],
        'location': location,
        'timezone': selection['timezone']['timeZoneId'],
        'latitude': selection['lat'],
        'longitude': selection['lng']
    });

    /* clear the text field and update the button state */
    document.getElementById('cityName').value = '';
    updateAddButtonState();

    /* save city list and update display */
    accessPrefs('write');
    accessPrefs('read');
    updateCityDisplay();
  }

  /* Possibly clean up the search state.  We don't want to do this if
   * the user clicked one of the sort buttons to reorder the results.
   * There are two ways to know when this happens: either
   * wasSortButtonPressed will be true or the popupCount will have
   * changed.  BTW, we delay 100ms to give any mouse events a chance
   * to fire first. */
  window.setTimeout(function() {
//    alert('after setting timeout');
    if (!wasSortButtonPressed && (popupCountStart == popupCount)) {
      clearSearchState();
    }
  }, 100);
}


/* Clean up the search state when we're done adding a city.  The msg
 * argument indicates what to display in the status area.  This should
 * be null if successful.  Otherwise it should be a very brief
 * sentence describing what went wrong. */
function clearSearchState(msg) {
//  alert('in clearSearchState()');
  setChildText(document.getElementById('statusMessage'), (msg) ? msg : '');
  isSearching = false;
  wasSortButtonPressed = false;
  popupCount = 0;
  document.getElementById('statusMessage').style.display = 'block';
  document.getElementById('sortControls').style.display = 'none';
  updateAddButtonState();
}


/* Handle row selection. */
function selectRow(index) {

  var table = document.getElementById('cityTable');
  var trList = table.getElementsByTagName('TR');

  /* adjust the colors of all of the rows */
  for (var i = 0; i < trList.length; ++i) {
    if ((i == index) && (i < cityList.length)) {
      trList[i].className = 'selected';
    }
    else {
      trList[i].className = (i % 2) ? 'evenRow' : 'oddRow';
    }
  }

  /* remember the selected city and update the remove button */
  selectedCityIndex = (index < cityList.length) ? index : null;
  removeButton.setEnabled(selectedCityIndex != null);
}


/* Remove a city from the list. */
function removeCity() {

  /* remove the city and deselect the row */
  cityList.splice(selectedCityIndex, 1);
  selectRow(null);

  /* update button state; if we had 7 cities before then we can enable
   * the controls now */
  updateAddButtonState();

  /* save city list and update display */
  accessPrefs('write');
  accessPrefs('read');
  updateCityDisplay();
}


/* Set the selected element's contained text.  We used to do this
 * lazily by setting el.textContent directly but that didn't work
 * under 10.4. */
function setChildText(el, text) {
  while (el.firstChild) {
    el.removeChild(el.firstChild);
  }
  el.appendChild(document.createTextNode(text));
}
